/*   Verse parser
 *   Magic Comprehensive Rules
 *   by ancestral
 *   
 *   Last updated 16 May 2013
 *
 */

module.exports = (function(){
  /*
   * Generated by PEG.js 0.7.0.
   *
   * http://pegjs.majda.cz/
   */
  
  function quote(s) {
    /*
     * ECMA-262, 5th ed., 7.8.4: All characters may appear literally in a
     * string literal except for the closing quote character, backslash,
     * carriage return, line separator, paragraph separator, and line feed.
     * Any character may appear in the form of an escape sequence.
     *
     * For portability, we also escape escape all control and non-ASCII
     * characters. Note that "\0" and "\v" escape sequences are not used
     * because JSHint does not like the first and IE the second.
     */
     return '"' + s
      .replace(/\\/g, '\\\\')  // backslash
      .replace(/"/g, '\\"')    // closing quote character
      .replace(/\x08/g, '\\b') // backspace
      .replace(/\t/g, '\\t')   // horizontal tab
      .replace(/\n/g, '\\n')   // line feed
      .replace(/\f/g, '\\f')   // form feed
      .replace(/\r/g, '\\r')   // carriage return
      .replace(/[\x00-\x07\x0B\x0E-\x1F\x80-\uFFFF]/g, escape)
      + '"';
  }
  
  var result = {
    /*
     * Parses the input with a generated parser. If the parsing is successfull,
     * returns a value explicitly or implicitly specified by the grammar from
     * which the parser was generated (see |PEG.buildParser|). If the parsing is
     * unsuccessful, throws |PEG.parser.SyntaxError| describing the error.
     */
    parse: function(input, startRule) {
      var parseFunctions = {
        "start": parse_start,
        "card": parse_card,
        "name": parse_name,
        "cost": parse_cost,
        "color": parse_color,
        "type": parse_type,
        "supertype": parse_supertype,
        "subtype": parse_subtype,
        "powtgh": parse_powtgh,
        "handlife": parse_handlife,
        "loyalty": parse_loyalty,
        "setrarity": parse_setrarity,
        "set": parse_set,
        "rarity": parse_rarity,
        "rules": parse_rules,
        "rule": parse_rule,
        "words": parse_words,
        "word": parse_word,
        "leftparen": parse_leftparen,
        "rightparen": parse_rightparen,
        "slash": parse_slash,
        "colon": parse_colon,
        "dash": parse_dash,
        "number": parse_number,
        "ws": parse_ws,
        "space": parse_space,
        "ret": parse_ret
      };
      
      if (startRule !== undefined) {
        if (parseFunctions[startRule] === undefined) {
          throw new Error("Invalid rule name: " + quote(startRule) + ".");
        }
      } else {
        startRule = "start";
      }
      
      var pos = 0;
      var reportFailures = 0;
      var rightmostFailuresPos = 0;
      var rightmostFailuresExpected = [];
      
      function padLeft(input, padding, length) {
        var result = input;
        
        var padLength = length - input.length;
        for (var i = 0; i < padLength; i++) {
          result = padding + result;
        }
        
        return result;
      }
      
      function escape(ch) {
        var charCode = ch.charCodeAt(0);
        var escapeChar;
        var length;
        
        if (charCode <= 0xFF) {
          escapeChar = 'x';
          length = 2;
        } else {
          escapeChar = 'u';
          length = 4;
        }
        
        return '\\' + escapeChar + padLeft(charCode.toString(16).toUpperCase(), '0', length);
      }
      
      function matchFailed(failure) {
        if (pos < rightmostFailuresPos) {
          return;
        }
        
        if (pos > rightmostFailuresPos) {
          rightmostFailuresPos = pos;
          rightmostFailuresExpected = [];
        }
        
        rightmostFailuresExpected.push(failure);
      }
      
      function parse_start() {
        var result0, result1;
        var pos0;
        
        pos0 = pos;
        result1 = parse_card();
        if (result1 !== null) {
          result0 = [];
          while (result1 !== null) {
            result0.push(result1);
            result1 = parse_card();
          }
        } else {
          result0 = null;
        }
        if (result0 !== null) {
          result0 = (function(offset, card) {return card})(pos0, result0);
        }
        if (result0 === null) {
          pos = pos0;
        }
        return result0;
      }
      
      function parse_card() {
        var result0, result1, result2, result3, result4, result5, result6, result7, result8;
        var pos0, pos1;
        
        pos0 = pos;
        pos1 = pos;
        result0 = parse_name();
        if (result0 !== null) {
          result1 = parse_cost();
          result1 = result1 !== null ? result1 : "";
          if (result1 !== null) {
            result2 = parse_color();
            result2 = result2 !== null ? result2 : "";
            if (result2 !== null) {
              result3 = parse_type();
              if (result3 !== null) {
                result4 = parse_handlife();
                result4 = result4 !== null ? result4 : "";
                if (result4 !== null) {
                  result5 = parse_loyalty();
                  result5 = result5 !== null ? result5 : "";
                  if (result5 !== null) {
                    result6 = parse_powtgh();
                    result6 = result6 !== null ? result6 : "";
                    if (result6 !== null) {
                      result7 = parse_rules();
                      if (result7 !== null) {
                        result8 = parse_setrarity();
                        if (result8 !== null) {
                          result0 = [result0, result1, result2, result3, result4, result5, result6, result7, result8];
                        } else {
                          result0 = null;
                          pos = pos1;
                        }
                      } else {
                        result0 = null;
                        pos = pos1;
                      }
                    } else {
                      result0 = null;
                      pos = pos1;
                    }
                  } else {
                    result0 = null;
                    pos = pos1;
                  }
                } else {
                  result0 = null;
                  pos = pos1;
                }
              } else {
                result0 = null;
                pos = pos1;
              }
            } else {
              result0 = null;
              pos = pos1;
            }
          } else {
            result0 = null;
            pos = pos1;
          }
        } else {
          result0 = null;
          pos = pos1;
        }
        if (result0 !== null) {
          result0 = (function(offset, name, cost, color, type, handlife, loyalty, powtgh, rules, setrarity) {
            var s = setrarity.join('');
            var o = { 
                name: name,
                cost: cost,
                type: type[1].join('').replace(/,\s*$/g, '').split(', '),
                rules: rules,
                setrarity: s.substring(0, s.length - 1).split(',')
            };
            if (color) {
              o.color = color;
            }
            if (type[0]) {
              o.supertype = type[0].join('').split(' ');
            }
            if (type[2]) {
              o.subtype = type[2].split(' ');
            }
            if (handlife) {
              o.hand = +handlife[0][1]
              o.life = +handlife[1][1]
            }
            if (loyalty) {
              o.loyalty = loyalty;
            }
            if (powtgh[3]) {
              o.power = +powtgh[3][0];
              o.toughness = +powtgh[3][1];
            }
            return o;
          })(pos0, result0[0], result0[1], result0[2], result0[3], result0[4], result0[5], result0[6], result0[7], result0[8]);
        }
        if (result0 === null) {
          pos = pos0;
        }
        return result0;
      }
      
      function parse_name() {
        var result0, result1, result2, result3, result4;
        var pos0, pos1;
        
        pos0 = pos;
        pos1 = pos;
        if (input.substr(pos, 4) === "Name") {
          result0 = "Name";
          pos += 4;
        } else {
          result0 = null;
          if (reportFailures === 0) {
            matchFailed("\"Name\"");
          }
        }
        if (result0 !== null) {
          result1 = parse_colon();
          result1 = result1 !== null ? result1 : "";
          if (result1 !== null) {
            result2 = parse_ws();
            if (result2 !== null) {
              result3 = parse_words();
              if (result3 !== null) {
                result4 = parse_ret();
                if (result4 !== null) {
                  result0 = [result0, result1, result2, result3, result4];
                } else {
                  result0 = null;
                  pos = pos1;
                }
              } else {
                result0 = null;
                pos = pos1;
              }
            } else {
              result0 = null;
              pos = pos1;
            }
          } else {
            result0 = null;
            pos = pos1;
          }
        } else {
          result0 = null;
          pos = pos1;
        }
        if (result0 !== null) {
          result0 = (function(offset, name) { return name })(pos0, result0[3]);
        }
        if (result0 === null) {
          pos = pos0;
        }
        return result0;
      }
      
      function parse_cost() {
        var result0, result1, result2, result3, result4;
        var pos0, pos1;
        
        pos0 = pos;
        pos1 = pos;
        if (input.substr(pos, 4) === "Cost") {
          result0 = "Cost";
          pos += 4;
        } else {
          result0 = null;
          if (reportFailures === 0) {
            matchFailed("\"Cost\"");
          }
        }
        if (result0 !== null) {
          result1 = parse_colon();
          result1 = result1 !== null ? result1 : "";
          if (result1 !== null) {
            result2 = parse_ws();
            if (result2 !== null) {
              result3 = parse_words();
              result3 = result3 !== null ? result3 : "";
              if (result3 !== null) {
                result4 = parse_ret();
                if (result4 !== null) {
                  result0 = [result0, result1, result2, result3, result4];
                } else {
                  result0 = null;
                  pos = pos1;
                }
              } else {
                result0 = null;
                pos = pos1;
              }
            } else {
              result0 = null;
              pos = pos1;
            }
          } else {
            result0 = null;
            pos = pos1;
          }
        } else {
          result0 = null;
          pos = pos1;
        }
        if (result0 !== null) {
          result0 = (function(offset, cost) { return cost })(pos0, result0[3]);
        }
        if (result0 === null) {
          pos = pos0;
        }
        return result0;
      }
      
      function parse_color() {
        var result0, result1, result2, result3, result4, result5;
        var pos0, pos1;
        
        pos0 = pos;
        pos1 = pos;
        if (input.substr(pos, 5) === "Color") {
          result0 = "Color";
          pos += 5;
        } else {
          result0 = null;
          if (reportFailures === 0) {
            matchFailed("\"Color\"");
          }
        }
        if (result0 !== null) {
          result1 = parse_colon();
          result1 = result1 !== null ? result1 : "";
          if (result1 !== null) {
            result2 = parse_ws();
            if (result2 !== null) {
              if (input.substr(pos, 4) === "Blue") {
                result4 = "Blue";
                pos += 4;
              } else {
                result4 = null;
                if (reportFailures === 0) {
                  matchFailed("\"Blue\"");
                }
              }
              if (result4 === null) {
                if (input.substr(pos, 5) === "Black") {
                  result4 = "Black";
                  pos += 5;
                } else {
                  result4 = null;
                  if (reportFailures === 0) {
                    matchFailed("\"Black\"");
                  }
                }
                if (result4 === null) {
                  if (input.substr(pos, 3) === "Red") {
                    result4 = "Red";
                    pos += 3;
                  } else {
                    result4 = null;
                    if (reportFailures === 0) {
                      matchFailed("\"Red\"");
                    }
                  }
                  if (result4 === null) {
                    if (input.substr(pos, 5) === "Green") {
                      result4 = "Green";
                      pos += 5;
                    } else {
                      result4 = null;
                      if (reportFailures === 0) {
                        matchFailed("\"Green\"");
                      }
                    }
                    if (result4 === null) {
                      if (input.substr(pos, 5) === "White") {
                        result4 = "White";
                        pos += 5;
                      } else {
                        result4 = null;
                        if (reportFailures === 0) {
                          matchFailed("\"White\"");
                        }
                      }
                      if (result4 === null) {
                        if (input.substr(pos, 9) === "Colorless") {
                          result4 = "Colorless";
                          pos += 9;
                        } else {
                          result4 = null;
                          if (reportFailures === 0) {
                            matchFailed("\"Colorless\"");
                          }
                        }
                        if (result4 === null) {
                          result4 = parse_slash();
                        }
                      }
                    }
                  }
                }
              }
              if (result4 !== null) {
                result3 = [];
                while (result4 !== null) {
                  result3.push(result4);
                  if (input.substr(pos, 4) === "Blue") {
                    result4 = "Blue";
                    pos += 4;
                  } else {
                    result4 = null;
                    if (reportFailures === 0) {
                      matchFailed("\"Blue\"");
                    }
                  }
                  if (result4 === null) {
                    if (input.substr(pos, 5) === "Black") {
                      result4 = "Black";
                      pos += 5;
                    } else {
                      result4 = null;
                      if (reportFailures === 0) {
                        matchFailed("\"Black\"");
                      }
                    }
                    if (result4 === null) {
                      if (input.substr(pos, 3) === "Red") {
                        result4 = "Red";
                        pos += 3;
                      } else {
                        result4 = null;
                        if (reportFailures === 0) {
                          matchFailed("\"Red\"");
                        }
                      }
                      if (result4 === null) {
                        if (input.substr(pos, 5) === "Green") {
                          result4 = "Green";
                          pos += 5;
                        } else {
                          result4 = null;
                          if (reportFailures === 0) {
                            matchFailed("\"Green\"");
                          }
                        }
                        if (result4 === null) {
                          if (input.substr(pos, 5) === "White") {
                            result4 = "White";
                            pos += 5;
                          } else {
                            result4 = null;
                            if (reportFailures === 0) {
                              matchFailed("\"White\"");
                            }
                          }
                          if (result4 === null) {
                            if (input.substr(pos, 9) === "Colorless") {
                              result4 = "Colorless";
                              pos += 9;
                            } else {
                              result4 = null;
                              if (reportFailures === 0) {
                                matchFailed("\"Colorless\"");
                              }
                            }
                            if (result4 === null) {
                              result4 = parse_slash();
                            }
                          }
                        }
                      }
                    }
                  }
                }
              } else {
                result3 = null;
              }
              if (result3 !== null) {
                result4 = parse_ws();
                result4 = result4 !== null ? result4 : "";
                if (result4 !== null) {
                  result5 = parse_ret();
                  if (result5 !== null) {
                    result0 = [result0, result1, result2, result3, result4, result5];
                  } else {
                    result0 = null;
                    pos = pos1;
                  }
                } else {
                  result0 = null;
                  pos = pos1;
                }
              } else {
                result0 = null;
                pos = pos1;
              }
            } else {
              result0 = null;
              pos = pos1;
            }
          } else {
            result0 = null;
            pos = pos1;
          }
        } else {
          result0 = null;
          pos = pos1;
        }
        if (result0 !== null) {
          result0 = (function(offset, color) { return color })(pos0, result0[3]);
        }
        if (result0 === null) {
          pos = pos0;
        }
        return result0;
      }
      
      function parse_type() {
        var result0, result1, result2, result3, result4, result5, result6, result7, result8;
        var pos0, pos1, pos2, pos3, pos4, pos5;
        
        pos0 = pos;
        pos1 = pos;
        if (input.substr(pos, 4) === "Type") {
          result0 = "Type";
          pos += 4;
        } else {
          result0 = null;
          if (reportFailures === 0) {
            matchFailed("\"Type\"");
          }
        }
        if (result0 !== null) {
          result1 = parse_colon();
          result1 = result1 !== null ? result1 : "";
          if (result1 !== null) {
            result2 = parse_ws();
            if (result2 !== null) {
              pos2 = pos;
              result3 = parse_supertype();
              if (result3 !== null) {
                result3 = (function(offset, supertype) {return supertype})(pos2, result3);
              }
              if (result3 === null) {
                pos = pos2;
              }
              result3 = result3 !== null ? result3 : "";
              if (result3 !== null) {
                pos2 = pos;
                if (input.substr(pos, 8) === "Artifact") {
                  result5 = "Artifact";
                  pos += 8;
                } else {
                  result5 = null;
                  if (reportFailures === 0) {
                    matchFailed("\"Artifact\"");
                  }
                }
                if (result5 === null) {
                  if (input.substr(pos, 8) === "Creature") {
                    result5 = "Creature";
                    pos += 8;
                  } else {
                    result5 = null;
                    if (reportFailures === 0) {
                      matchFailed("\"Creature\"");
                    }
                  }
                  if (result5 === null) {
                    if (input.substr(pos, 11) === "Enchantment") {
                      result5 = "Enchantment";
                      pos += 11;
                    } else {
                      result5 = null;
                      if (reportFailures === 0) {
                        matchFailed("\"Enchantment\"");
                      }
                    }
                    if (result5 === null) {
                      if (input.substr(pos, 7) === "Instant") {
                        result5 = "Instant";
                        pos += 7;
                      } else {
                        result5 = null;
                        if (reportFailures === 0) {
                          matchFailed("\"Instant\"");
                        }
                      }
                      if (result5 === null) {
                        if (input.substr(pos, 4) === "Land") {
                          result5 = "Land";
                          pos += 4;
                        } else {
                          result5 = null;
                          if (reportFailures === 0) {
                            matchFailed("\"Land\"");
                          }
                        }
                        if (result5 === null) {
                          if (input.substr(pos, 10) === "Phenomenon") {
                            result5 = "Phenomenon";
                            pos += 10;
                          } else {
                            result5 = null;
                            if (reportFailures === 0) {
                              matchFailed("\"Phenomenon\"");
                            }
                          }
                          if (result5 === null) {
                            if (input.substr(pos, 12) === "Planeswalker") {
                              result5 = "Planeswalker";
                              pos += 12;
                            } else {
                              result5 = null;
                              if (reportFailures === 0) {
                                matchFailed("\"Planeswalker\"");
                              }
                            }
                            if (result5 === null) {
                              if (input.substr(pos, 5) === "Plane") {
                                result5 = "Plane";
                                pos += 5;
                              } else {
                                result5 = null;
                                if (reportFailures === 0) {
                                  matchFailed("\"Plane\"");
                                }
                              }
                              if (result5 === null) {
                                if (input.substr(pos, 6) === "Scheme") {
                                  result5 = "Scheme";
                                  pos += 6;
                                } else {
                                  result5 = null;
                                  if (reportFailures === 0) {
                                    matchFailed("\"Scheme\"");
                                  }
                                }
                                if (result5 === null) {
                                  if (input.substr(pos, 7) === "Sorcery") {
                                    result5 = "Sorcery";
                                    pos += 7;
                                  } else {
                                    result5 = null;
                                    if (reportFailures === 0) {
                                      matchFailed("\"Sorcery\"");
                                    }
                                  }
                                  if (result5 === null) {
                                    if (input.substr(pos, 6) === "Tribal") {
                                      result5 = "Tribal";
                                      pos += 6;
                                    } else {
                                      result5 = null;
                                      if (reportFailures === 0) {
                                        matchFailed("\"Tribal\"");
                                      }
                                    }
                                    if (result5 === null) {
                                      if (input.substr(pos, 8) === "Vanguard") {
                                        result5 = "Vanguard";
                                        pos += 8;
                                      } else {
                                        result5 = null;
                                        if (reportFailures === 0) {
                                          matchFailed("\"Vanguard\"");
                                        }
                                      }
                                    }
                                  }
                                }
                              }
                            }
                          }
                        }
                      }
                    }
                  }
                }
                if (result5 !== null) {
                  result6 = parse_ws();
                  result6 = result6 !== null ? result6 : "";
                  if (result6 !== null) {
                    result5 = [result5, result6];
                  } else {
                    result5 = null;
                    pos = pos2;
                  }
                } else {
                  result5 = null;
                  pos = pos2;
                }
                if (result5 !== null) {
                  result4 = [];
                  while (result5 !== null) {
                    result4.push(result5);
                    pos2 = pos;
                    if (input.substr(pos, 8) === "Artifact") {
                      result5 = "Artifact";
                      pos += 8;
                    } else {
                      result5 = null;
                      if (reportFailures === 0) {
                        matchFailed("\"Artifact\"");
                      }
                    }
                    if (result5 === null) {
                      if (input.substr(pos, 8) === "Creature") {
                        result5 = "Creature";
                        pos += 8;
                      } else {
                        result5 = null;
                        if (reportFailures === 0) {
                          matchFailed("\"Creature\"");
                        }
                      }
                      if (result5 === null) {
                        if (input.substr(pos, 11) === "Enchantment") {
                          result5 = "Enchantment";
                          pos += 11;
                        } else {
                          result5 = null;
                          if (reportFailures === 0) {
                            matchFailed("\"Enchantment\"");
                          }
                        }
                        if (result5 === null) {
                          if (input.substr(pos, 7) === "Instant") {
                            result5 = "Instant";
                            pos += 7;
                          } else {
                            result5 = null;
                            if (reportFailures === 0) {
                              matchFailed("\"Instant\"");
                            }
                          }
                          if (result5 === null) {
                            if (input.substr(pos, 4) === "Land") {
                              result5 = "Land";
                              pos += 4;
                            } else {
                              result5 = null;
                              if (reportFailures === 0) {
                                matchFailed("\"Land\"");
                              }
                            }
                            if (result5 === null) {
                              if (input.substr(pos, 10) === "Phenomenon") {
                                result5 = "Phenomenon";
                                pos += 10;
                              } else {
                                result5 = null;
                                if (reportFailures === 0) {
                                  matchFailed("\"Phenomenon\"");
                                }
                              }
                              if (result5 === null) {
                                if (input.substr(pos, 12) === "Planeswalker") {
                                  result5 = "Planeswalker";
                                  pos += 12;
                                } else {
                                  result5 = null;
                                  if (reportFailures === 0) {
                                    matchFailed("\"Planeswalker\"");
                                  }
                                }
                                if (result5 === null) {
                                  if (input.substr(pos, 5) === "Plane") {
                                    result5 = "Plane";
                                    pos += 5;
                                  } else {
                                    result5 = null;
                                    if (reportFailures === 0) {
                                      matchFailed("\"Plane\"");
                                    }
                                  }
                                  if (result5 === null) {
                                    if (input.substr(pos, 6) === "Scheme") {
                                      result5 = "Scheme";
                                      pos += 6;
                                    } else {
                                      result5 = null;
                                      if (reportFailures === 0) {
                                        matchFailed("\"Scheme\"");
                                      }
                                    }
                                    if (result5 === null) {
                                      if (input.substr(pos, 7) === "Sorcery") {
                                        result5 = "Sorcery";
                                        pos += 7;
                                      } else {
                                        result5 = null;
                                        if (reportFailures === 0) {
                                          matchFailed("\"Sorcery\"");
                                        }
                                      }
                                      if (result5 === null) {
                                        if (input.substr(pos, 6) === "Tribal") {
                                          result5 = "Tribal";
                                          pos += 6;
                                        } else {
                                          result5 = null;
                                          if (reportFailures === 0) {
                                            matchFailed("\"Tribal\"");
                                          }
                                        }
                                        if (result5 === null) {
                                          if (input.substr(pos, 8) === "Vanguard") {
                                            result5 = "Vanguard";
                                            pos += 8;
                                          } else {
                                            result5 = null;
                                            if (reportFailures === 0) {
                                              matchFailed("\"Vanguard\"");
                                            }
                                          }
                                        }
                                      }
                                    }
                                  }
                                }
                              }
                            }
                          }
                        }
                      }
                    }
                    if (result5 !== null) {
                      result6 = parse_ws();
                      result6 = result6 !== null ? result6 : "";
                      if (result6 !== null) {
                        result5 = [result5, result6];
                      } else {
                        result5 = null;
                        pos = pos2;
                      }
                    } else {
                      result5 = null;
                      pos = pos2;
                    }
                  }
                } else {
                  result4 = null;
                }
                if (result4 !== null) {
                  pos2 = pos;
                  pos3 = pos;
                  result5 = parse_dash();
                  if (result5 !== null) {
                    result6 = [];
                    pos4 = pos;
                    pos5 = pos;
                    result7 = parse_ws();
                    if (result7 !== null) {
                      result8 = parse_subtype();
                      if (result8 !== null) {
                        result7 = [result7, result8];
                      } else {
                        result7 = null;
                        pos = pos5;
                      }
                    } else {
                      result7 = null;
                      pos = pos5;
                    }
                    if (result7 !== null) {
                      result7 = (function(offset, subtype) {return subtype})(pos4, result7[1]);
                    }
                    if (result7 === null) {
                      pos = pos4;
                    }
                    while (result7 !== null) {
                      result6.push(result7);
                      pos4 = pos;
                      pos5 = pos;
                      result7 = parse_ws();
                      if (result7 !== null) {
                        result8 = parse_subtype();
                        if (result8 !== null) {
                          result7 = [result7, result8];
                        } else {
                          result7 = null;
                          pos = pos5;
                        }
                      } else {
                        result7 = null;
                        pos = pos5;
                      }
                      if (result7 !== null) {
                        result7 = (function(offset, subtype) {return subtype})(pos4, result7[1]);
                      }
                      if (result7 === null) {
                        pos = pos4;
                      }
                    }
                    if (result6 !== null) {
                      result5 = [result5, result6];
                    } else {
                      result5 = null;
                      pos = pos3;
                    }
                  } else {
                    result5 = null;
                    pos = pos3;
                  }
                  if (result5 !== null) {
                    result5 = (function(offset, subtype) {return subtype.join('') })(pos2, result5[1]);
                  }
                  if (result5 === null) {
                    pos = pos2;
                  }
                  result5 = result5 !== null ? result5 : "";
                  if (result5 !== null) {
                    result6 = parse_ret();
                    if (result6 !== null) {
                      result0 = [result0, result1, result2, result3, result4, result5, result6];
                    } else {
                      result0 = null;
                      pos = pos1;
                    }
                  } else {
                    result0 = null;
                    pos = pos1;
                  }
                } else {
                  result0 = null;
                  pos = pos1;
                }
              } else {
                result0 = null;
                pos = pos1;
              }
            } else {
              result0 = null;
              pos = pos1;
            }
          } else {
            result0 = null;
            pos = pos1;
          }
        } else {
          result0 = null;
          pos = pos1;
        }
        if (result0 !== null) {
          result0 = (function(offset, supertype, type, subtype) { return [supertype, type, subtype.substring(0, subtype.length - 1)] })(pos0, result0[3], result0[4], result0[5]);
        }
        if (result0 === null) {
          pos = pos0;
        }
        return result0;
      }
      
      function parse_supertype() {
        var result0, result1, result2;
        var pos0, pos1;
        
        pos0 = pos;
        pos1 = pos;
        if (input.substr(pos, 5) === "Basic") {
          result1 = "Basic";
          pos += 5;
        } else {
          result1 = null;
          if (reportFailures === 0) {
            matchFailed("\"Basic\"");
          }
        }
        if (result1 === null) {
          if (input.substr(pos, 9) === "Legendary") {
            result1 = "Legendary";
            pos += 9;
          } else {
            result1 = null;
            if (reportFailures === 0) {
              matchFailed("\"Legendary\"");
            }
          }
          if (result1 === null) {
            if (input.substr(pos, 7) === "Ongoing") {
              result1 = "Ongoing";
              pos += 7;
            } else {
              result1 = null;
              if (reportFailures === 0) {
                matchFailed("\"Ongoing\"");
              }
            }
            if (result1 === null) {
              if (input.substr(pos, 4) === "Snow") {
                result1 = "Snow";
                pos += 4;
              } else {
                result1 = null;
                if (reportFailures === 0) {
                  matchFailed("\"Snow\"");
                }
              }
              if (result1 === null) {
                if (input.substr(pos, 5) === "World") {
                  result1 = "World";
                  pos += 5;
                } else {
                  result1 = null;
                  if (reportFailures === 0) {
                    matchFailed("\"World\"");
                  }
                }
              }
            }
          }
        }
        if (result1 !== null) {
          result2 = parse_ws();
          result2 = result2 !== null ? result2 : "";
          if (result2 !== null) {
            result1 = [result1, result2];
          } else {
            result1 = null;
            pos = pos1;
          }
        } else {
          result1 = null;
          pos = pos1;
        }
        if (result1 !== null) {
          result0 = [];
          while (result1 !== null) {
            result0.push(result1);
            pos1 = pos;
            if (input.substr(pos, 5) === "Basic") {
              result1 = "Basic";
              pos += 5;
            } else {
              result1 = null;
              if (reportFailures === 0) {
                matchFailed("\"Basic\"");
              }
            }
            if (result1 === null) {
              if (input.substr(pos, 9) === "Legendary") {
                result1 = "Legendary";
                pos += 9;
              } else {
                result1 = null;
                if (reportFailures === 0) {
                  matchFailed("\"Legendary\"");
                }
              }
              if (result1 === null) {
                if (input.substr(pos, 7) === "Ongoing") {
                  result1 = "Ongoing";
                  pos += 7;
                } else {
                  result1 = null;
                  if (reportFailures === 0) {
                    matchFailed("\"Ongoing\"");
                  }
                }
                if (result1 === null) {
                  if (input.substr(pos, 4) === "Snow") {
                    result1 = "Snow";
                    pos += 4;
                  } else {
                    result1 = null;
                    if (reportFailures === 0) {
                      matchFailed("\"Snow\"");
                    }
                  }
                  if (result1 === null) {
                    if (input.substr(pos, 5) === "World") {
                      result1 = "World";
                      pos += 5;
                    } else {
                      result1 = null;
                      if (reportFailures === 0) {
                        matchFailed("\"World\"");
                      }
                    }
                  }
                }
              }
            }
            if (result1 !== null) {
              result2 = parse_ws();
              result2 = result2 !== null ? result2 : "";
              if (result2 !== null) {
                result1 = [result1, result2];
              } else {
                result1 = null;
                pos = pos1;
              }
            } else {
              result1 = null;
              pos = pos1;
            }
          }
        } else {
          result0 = null;
        }
        if (result0 !== null) {
          result0 = (function(offset, supertype) { return supertype })(pos0, result0);
        }
        if (result0 === null) {
          pos = pos0;
        }
        return result0;
      }
      
      function parse_subtype() {
        var result0, result1, result2, result3;
        var pos0, pos1;
        
        pos0 = pos;
        result0 = parse_words();
        if (result0 !== null) {
          result1 = [];
          pos1 = pos;
          result2 = parse_ws();
          if (result2 !== null) {
            result3 = parse_words();
            if (result3 !== null) {
              result2 = [result2, result3];
            } else {
              result2 = null;
              pos = pos1;
            }
          } else {
            result2 = null;
            pos = pos1;
          }
          while (result2 !== null) {
            result1.push(result2);
            pos1 = pos;
            result2 = parse_ws();
            if (result2 !== null) {
              result3 = parse_words();
              if (result3 !== null) {
                result2 = [result2, result3];
              } else {
                result2 = null;
                pos = pos1;
              }
            } else {
              result2 = null;
              pos = pos1;
            }
          }
          if (result1 !== null) {
            result0 = [result0, result1];
          } else {
            result0 = null;
            pos = pos0;
          }
        } else {
          result0 = null;
          pos = pos0;
        }
        return result0;
      }
      
      function parse_powtgh() {
        var result0, result1, result2, result3, result4, result5, result6, result7;
        var pos0, pos1, pos2;
        
        pos0 = pos;
        if (input.substr(pos, 7) === "Pow/Tgh") {
          result0 = "Pow/Tgh";
          pos += 7;
        } else {
          result0 = null;
          if (reportFailures === 0) {
            matchFailed("\"Pow/Tgh\"");
          }
        }
        if (result0 !== null) {
          result1 = parse_colon();
          result1 = result1 !== null ? result1 : "";
          if (result1 !== null) {
            result2 = parse_ws();
            if (result2 !== null) {
              pos1 = pos;
              pos2 = pos;
              result3 = parse_leftparen();
              if (result3 !== null) {
                result4 = parse_number();
                if (result4 !== null) {
                  result5 = parse_slash();
                  if (result5 !== null) {
                    result6 = parse_number();
                    if (result6 !== null) {
                      result7 = parse_rightparen();
                      if (result7 !== null) {
                        result3 = [result3, result4, result5, result6, result7];
                      } else {
                        result3 = null;
                        pos = pos2;
                      }
                    } else {
                      result3 = null;
                      pos = pos2;
                    }
                  } else {
                    result3 = null;
                    pos = pos2;
                  }
                } else {
                  result3 = null;
                  pos = pos2;
                }
              } else {
                result3 = null;
                pos = pos2;
              }
              if (result3 !== null) {
                result3 = (function(offset, power, toughness) { return [ power, toughness ] })(pos1, result3[1], result3[3]);
              }
              if (result3 === null) {
                pos = pos1;
              }
              result3 = result3 !== null ? result3 : "";
              if (result3 !== null) {
                result4 = parse_ret();
                if (result4 !== null) {
                  result0 = [result0, result1, result2, result3, result4];
                } else {
                  result0 = null;
                  pos = pos0;
                }
              } else {
                result0 = null;
                pos = pos0;
              }
            } else {
              result0 = null;
              pos = pos0;
            }
          } else {
            result0 = null;
            pos = pos0;
          }
        } else {
          result0 = null;
          pos = pos0;
        }
        return result0;
      }
      
      function parse_handlife() {
        var result0, result1, result2, result3, result4, result5, result6, result7, result8, result9;
        var pos0, pos1, pos2;
        
        pos0 = pos;
        pos1 = pos;
        if (input.substr(pos, 9) === "Hand/Life") {
          result0 = "Hand/Life";
          pos += 9;
        } else {
          result0 = null;
          if (reportFailures === 0) {
            matchFailed("\"Hand/Life\"");
          }
        }
        if (result0 !== null) {
          result1 = parse_colon();
          result1 = result1 !== null ? result1 : "";
          if (result1 !== null) {
            result2 = parse_ws();
            if (result2 !== null) {
              if (input.substr(pos, 15) === "Hand Modifier: ") {
                result3 = "Hand Modifier: ";
                pos += 15;
              } else {
                result3 = null;
                if (reportFailures === 0) {
                  matchFailed("\"Hand Modifier: \"");
                }
              }
              if (result3 !== null) {
                pos2 = pos;
                if (/^[+\-]/.test(input.charAt(pos))) {
                  result4 = input.charAt(pos);
                  pos++;
                } else {
                  result4 = null;
                  if (reportFailures === 0) {
                    matchFailed("[+\\-]");
                  }
                }
                if (result4 !== null) {
                  result5 = parse_number();
                  if (result5 !== null) {
                    result4 = [result4, result5];
                  } else {
                    result4 = null;
                    pos = pos2;
                  }
                } else {
                  result4 = null;
                  pos = pos2;
                }
                if (result4 !== null) {
                  if (/^[ ,]/.test(input.charAt(pos))) {
                    result6 = input.charAt(pos);
                    pos++;
                  } else {
                    result6 = null;
                    if (reportFailures === 0) {
                      matchFailed("[ ,]");
                    }
                  }
                  if (result6 !== null) {
                    result5 = [];
                    while (result6 !== null) {
                      result5.push(result6);
                      if (/^[ ,]/.test(input.charAt(pos))) {
                        result6 = input.charAt(pos);
                        pos++;
                      } else {
                        result6 = null;
                        if (reportFailures === 0) {
                          matchFailed("[ ,]");
                        }
                      }
                    }
                  } else {
                    result5 = null;
                  }
                  if (result5 !== null) {
                    if (input.substr(pos, 15) === "Life Modifier: ") {
                      result6 = "Life Modifier: ";
                      pos += 15;
                    } else {
                      result6 = null;
                      if (reportFailures === 0) {
                        matchFailed("\"Life Modifier: \"");
                      }
                    }
                    if (result6 !== null) {
                      pos2 = pos;
                      if (/^[+\-]/.test(input.charAt(pos))) {
                        result7 = input.charAt(pos);
                        pos++;
                      } else {
                        result7 = null;
                        if (reportFailures === 0) {
                          matchFailed("[+\\-]");
                        }
                      }
                      result7 = result7 !== null ? result7 : "";
                      if (result7 !== null) {
                        result8 = parse_number();
                        if (result8 !== null) {
                          result7 = [result7, result8];
                        } else {
                          result7 = null;
                          pos = pos2;
                        }
                      } else {
                        result7 = null;
                        pos = pos2;
                      }
                      if (result7 !== null) {
                        result8 = parse_ws();
                        result8 = result8 !== null ? result8 : "";
                        if (result8 !== null) {
                          result9 = parse_ret();
                          if (result9 !== null) {
                            result0 = [result0, result1, result2, result3, result4, result5, result6, result7, result8, result9];
                          } else {
                            result0 = null;
                            pos = pos1;
                          }
                        } else {
                          result0 = null;
                          pos = pos1;
                        }
                      } else {
                        result0 = null;
                        pos = pos1;
                      }
                    } else {
                      result0 = null;
                      pos = pos1;
                    }
                  } else {
                    result0 = null;
                    pos = pos1;
                  }
                } else {
                  result0 = null;
                  pos = pos1;
                }
              } else {
                result0 = null;
                pos = pos1;
              }
            } else {
              result0 = null;
              pos = pos1;
            }
          } else {
            result0 = null;
            pos = pos1;
          }
        } else {
          result0 = null;
          pos = pos1;
        }
        if (result0 !== null) {
          result0 = (function(offset, hand, life) { return [ hand, life ] })(pos0, result0[4], result0[7]);
        }
        if (result0 === null) {
          pos = pos0;
        }
        return result0;
      }
      
      function parse_loyalty() {
        var result0, result1, result2, result3, result4, result5, result6;
        var pos0, pos1;
        
        pos0 = pos;
        pos1 = pos;
        if (input.substr(pos, 7) === "Loyalty") {
          result0 = "Loyalty";
          pos += 7;
        } else {
          result0 = null;
          if (reportFailures === 0) {
            matchFailed("\"Loyalty\"");
          }
        }
        if (result0 !== null) {
          result1 = parse_colon();
          result1 = result1 !== null ? result1 : "";
          if (result1 !== null) {
            result2 = parse_ws();
            if (result2 !== null) {
              result3 = parse_leftparen();
              if (result3 !== null) {
                result4 = parse_number();
                if (result4 !== null) {
                  result5 = parse_rightparen();
                  if (result5 !== null) {
                    result6 = parse_ret();
                    if (result6 !== null) {
                      result0 = [result0, result1, result2, result3, result4, result5, result6];
                    } else {
                      result0 = null;
                      pos = pos1;
                    }
                  } else {
                    result0 = null;
                    pos = pos1;
                  }
                } else {
                  result0 = null;
                  pos = pos1;
                }
              } else {
                result0 = null;
                pos = pos1;
              }
            } else {
              result0 = null;
              pos = pos1;
            }
          } else {
            result0 = null;
            pos = pos1;
          }
        } else {
          result0 = null;
          pos = pos1;
        }
        if (result0 !== null) {
          result0 = (function(offset, loyalty) { return loyalty })(pos0, result0[4]);
        }
        if (result0 === null) {
          pos = pos0;
        }
        return result0;
      }
      
      function parse_setrarity() {
        var result0, result1, result2, result3, result4, result5, result6, result7;
        var pos0, pos1, pos2, pos3;
        
        pos0 = pos;
        pos1 = pos;
        if (input.substr(pos, 10) === "Set/Rarity") {
          result0 = "Set/Rarity";
          pos += 10;
        } else {
          result0 = null;
          if (reportFailures === 0) {
            matchFailed("\"Set/Rarity\"");
          }
        }
        if (result0 !== null) {
          result1 = parse_colon();
          result1 = result1 !== null ? result1 : "";
          if (result1 !== null) {
            result2 = parse_ws();
            if (result2 !== null) {
              result3 = [];
              pos2 = pos;
              pos3 = pos;
              if (input.substr(pos, 2) === ", ") {
                result4 = ", ";
                pos += 2;
              } else {
                result4 = null;
                if (reportFailures === 0) {
                  matchFailed("\", \"");
                }
              }
              result4 = result4 !== null ? result4 : "";
              if (result4 !== null) {
                result5 = parse_set();
                if (result5 !== null) {
                  result6 = parse_ws();
                  if (result6 !== null) {
                    result7 = parse_rarity();
                    if (result7 !== null) {
                      result4 = [result4, result5, result6, result7];
                    } else {
                      result4 = null;
                      pos = pos3;
                    }
                  } else {
                    result4 = null;
                    pos = pos3;
                  }
                } else {
                  result4 = null;
                  pos = pos3;
                }
              } else {
                result4 = null;
                pos = pos3;
              }
              if (result4 !== null) {
                result4 = (function(offset, set, rarity) { return set + ' ' + rarity + ','})(pos2, result4[1], result4[3]);
              }
              if (result4 === null) {
                pos = pos2;
              }
              while (result4 !== null) {
                result3.push(result4);
                pos2 = pos;
                pos3 = pos;
                if (input.substr(pos, 2) === ", ") {
                  result4 = ", ";
                  pos += 2;
                } else {
                  result4 = null;
                  if (reportFailures === 0) {
                    matchFailed("\", \"");
                  }
                }
                result4 = result4 !== null ? result4 : "";
                if (result4 !== null) {
                  result5 = parse_set();
                  if (result5 !== null) {
                    result6 = parse_ws();
                    if (result6 !== null) {
                      result7 = parse_rarity();
                      if (result7 !== null) {
                        result4 = [result4, result5, result6, result7];
                      } else {
                        result4 = null;
                        pos = pos3;
                      }
                    } else {
                      result4 = null;
                      pos = pos3;
                    }
                  } else {
                    result4 = null;
                    pos = pos3;
                  }
                } else {
                  result4 = null;
                  pos = pos3;
                }
                if (result4 !== null) {
                  result4 = (function(offset, set, rarity) { return set + ' ' + rarity + ','})(pos2, result4[1], result4[3]);
                }
                if (result4 === null) {
                  pos = pos2;
                }
              }
              if (result3 !== null) {
                result4 = parse_ret();
                if (result4 !== null) {
                  result0 = [result0, result1, result2, result3, result4];
                } else {
                  result0 = null;
                  pos = pos1;
                }
              } else {
                result0 = null;
                pos = pos1;
              }
            } else {
              result0 = null;
              pos = pos1;
            }
          } else {
            result0 = null;
            pos = pos1;
          }
        } else {
          result0 = null;
          pos = pos1;
        }
        if (result0 !== null) {
          result0 = (function(offset, sets) { return sets })(pos0, result0[3]);
        }
        if (result0 === null) {
          pos = pos0;
        }
        return result0;
      }
      
      function parse_set() {
        var result0;
        
        if (input.substr(pos, 10) === "Magic 2013") {
          result0 = "Magic 2013";
          pos += 10;
        } else {
          result0 = null;
          if (reportFailures === 0) {
            matchFailed("\"Magic 2013\"");
          }
        }
        if (result0 === null) {
          if (input.substr(pos, 10) === "Magic 2012") {
            result0 = "Magic 2012";
            pos += 10;
          } else {
            result0 = null;
            if (reportFailures === 0) {
              matchFailed("\"Magic 2012\"");
            }
          }
          if (result0 === null) {
            if (input.substr(pos, 10) === "Magic 2011") {
              result0 = "Magic 2011";
              pos += 10;
            } else {
              result0 = null;
              if (reportFailures === 0) {
                matchFailed("\"Magic 2011\"");
              }
            }
            if (result0 === null) {
              if (input.substr(pos, 10) === "Magic 2010") {
                result0 = "Magic 2010";
                pos += 10;
              } else {
                result0 = null;
                if (reportFailures === 0) {
                  matchFailed("\"Magic 2010\"");
                }
              }
              if (result0 === null) {
                if (input.substr(pos, 13) === "Tenth Edition") {
                  result0 = "Tenth Edition";
                  pos += 13;
                } else {
                  result0 = null;
                  if (reportFailures === 0) {
                    matchFailed("\"Tenth Edition\"");
                  }
                }
                if (result0 === null) {
                  if (input.substr(pos, 13) === "Ninth Edition") {
                    result0 = "Ninth Edition";
                    pos += 13;
                  } else {
                    result0 = null;
                    if (reportFailures === 0) {
                      matchFailed("\"Ninth Edition\"");
                    }
                  }
                  if (result0 === null) {
                    if (input.substr(pos, 14) === "Eighth Edition") {
                      result0 = "Eighth Edition";
                      pos += 14;
                    } else {
                      result0 = null;
                      if (reportFailures === 0) {
                        matchFailed("\"Eighth Edition\"");
                      }
                    }
                    if (result0 === null) {
                      if (input.substr(pos, 15) === "Seventh Edition") {
                        result0 = "Seventh Edition";
                        pos += 15;
                      } else {
                        result0 = null;
                        if (reportFailures === 0) {
                          matchFailed("\"Seventh Edition\"");
                        }
                      }
                      if (result0 === null) {
                        if (input.substr(pos, 21) === "Classic Sixth Edition") {
                          result0 = "Classic Sixth Edition";
                          pos += 21;
                        } else {
                          result0 = null;
                          if (reportFailures === 0) {
                            matchFailed("\"Classic Sixth Edition\"");
                          }
                        }
                        if (result0 === null) {
                          if (input.substr(pos, 13) === "Fifth Edition") {
                            result0 = "Fifth Edition";
                            pos += 13;
                          } else {
                            result0 = null;
                            if (reportFailures === 0) {
                              matchFailed("\"Fifth Edition\"");
                            }
                          }
                          if (result0 === null) {
                            if (input.substr(pos, 14) === "Fourth Edition") {
                              result0 = "Fourth Edition";
                              pos += 14;
                            } else {
                              result0 = null;
                              if (reportFailures === 0) {
                                matchFailed("\"Fourth Edition\"");
                              }
                            }
                            if (result0 === null) {
                              if (input.substr(pos, 15) === "Revised Edition") {
                                result0 = "Revised Edition";
                                pos += 15;
                              } else {
                                result0 = null;
                                if (reportFailures === 0) {
                                  matchFailed("\"Revised Edition\"");
                                }
                              }
                              if (result0 === null) {
                                if (input.substr(pos, 17) === "Unlimited Edition") {
                                  result0 = "Unlimited Edition";
                                  pos += 17;
                                } else {
                                  result0 = null;
                                  if (reportFailures === 0) {
                                    matchFailed("\"Unlimited Edition\"");
                                  }
                                }
                                if (result0 === null) {
                                  if (input.substr(pos, 20) === "Limited Edition Beta") {
                                    result0 = "Limited Edition Beta";
                                    pos += 20;
                                  } else {
                                    result0 = null;
                                    if (reportFailures === 0) {
                                      matchFailed("\"Limited Edition Beta\"");
                                    }
                                  }
                                  if (result0 === null) {
                                    if (input.substr(pos, 21) === "Limited Edition Alpha") {
                                      result0 = "Limited Edition Alpha";
                                      pos += 21;
                                    } else {
                                      result0 = null;
                                      if (reportFailures === 0) {
                                        matchFailed("\"Limited Edition Alpha\"");
                                      }
                                    }
                                    if (result0 === null) {
                                      if (input.substr(pos, 18) === "Masters Edition IV") {
                                        result0 = "Masters Edition IV";
                                        pos += 18;
                                      } else {
                                        result0 = null;
                                        if (reportFailures === 0) {
                                          matchFailed("\"Masters Edition IV\"");
                                        }
                                      }
                                      if (result0 === null) {
                                        if (input.substr(pos, 19) === "Masters Edition III") {
                                          result0 = "Masters Edition III";
                                          pos += 19;
                                        } else {
                                          result0 = null;
                                          if (reportFailures === 0) {
                                            matchFailed("\"Masters Edition III\"");
                                          }
                                        }
                                        if (result0 === null) {
                                          if (input.substr(pos, 18) === "Masters Edition II") {
                                            result0 = "Masters Edition II";
                                            pos += 18;
                                          } else {
                                            result0 = null;
                                            if (reportFailures === 0) {
                                              matchFailed("\"Masters Edition II\"");
                                            }
                                          }
                                          if (result0 === null) {
                                            if (input.substr(pos, 15) === "Masters Edition") {
                                              result0 = "Masters Edition";
                                              pos += 15;
                                            } else {
                                              result0 = null;
                                              if (reportFailures === 0) {
                                                matchFailed("\"Masters Edition\"");
                                              }
                                            }
                                            if (result0 === null) {
                                              if (input.substr(pos, 14) === "Modern Masters") {
                                                result0 = "Modern Masters";
                                                pos += 14;
                                              } else {
                                                result0 = null;
                                                if (reportFailures === 0) {
                                                  matchFailed("\"Modern Masters\"");
                                                }
                                              }
                                              if (result0 === null) {
                                                if (input.substr(pos, 8) === "Unhinged") {
                                                  result0 = "Unhinged";
                                                  pos += 8;
                                                } else {
                                                  result0 = null;
                                                  if (reportFailures === 0) {
                                                    matchFailed("\"Unhinged\"");
                                                  }
                                                }
                                                if (result0 === null) {
                                                  if (input.substr(pos, 7) === "Unglued") {
                                                    result0 = "Unglued";
                                                    pos += 7;
                                                  } else {
                                                    result0 = null;
                                                    if (reportFailures === 0) {
                                                      matchFailed("\"Unglued\"");
                                                    }
                                                  }
                                                  if (result0 === null) {
                                                    if (input.substr(pos, 8) === "Vanguard") {
                                                      result0 = "Vanguard";
                                                      pos += 8;
                                                    } else {
                                                      result0 = null;
                                                      if (reportFailures === 0) {
                                                        matchFailed("\"Vanguard\"");
                                                      }
                                                    }
                                                    if (result0 === null) {
                                                      if (input.substr(pos, 32) === "Deckmasters: Garfield vs. Finkel") {
                                                        result0 = "Deckmasters: Garfield vs. Finkel";
                                                        pos += 32;
                                                      } else {
                                                        result0 = null;
                                                        if (reportFailures === 0) {
                                                          matchFailed("\"Deckmasters: Garfield vs. Finkel\"");
                                                        }
                                                      }
                                                      if (result0 === null) {
                                                        if (input.substr(pos, 16) === "Beatdown Box Set") {
                                                          result0 = "Beatdown Box Set";
                                                          pos += 16;
                                                        } else {
                                                          result0 = null;
                                                          if (reportFailures === 0) {
                                                            matchFailed("\"Beatdown Box Set\"");
                                                          }
                                                        }
                                                        if (result0 === null) {
                                                          if (input.substr(pos, 19) === "Commander's Arsenal") {
                                                            result0 = "Commander's Arsenal";
                                                            pos += 19;
                                                          } else {
                                                            result0 = null;
                                                            if (reportFailures === 0) {
                                                              matchFailed("\"Commander's Arsenal\"");
                                                            }
                                                          }
                                                          if (result0 === null) {
                                                            if (input.substr(pos, 30) === "Duel Decks: Divine vs. Demonic") {
                                                              result0 = "Duel Decks: Divine vs. Demonic";
                                                              pos += 30;
                                                            } else {
                                                              result0 = null;
                                                              if (reportFailures === 0) {
                                                                matchFailed("\"Duel Decks: Divine vs. Demonic\"");
                                                              }
                                                            }
                                                            if (result0 === null) {
                                                              if (input.substr(pos, 27) === "Duel Decks: Venser vs. Koth") {
                                                                result0 = "Duel Decks: Venser vs. Koth";
                                                                pos += 27;
                                                              } else {
                                                                result0 = null;
                                                                if (reportFailures === 0) {
                                                                  matchFailed("\"Duel Decks: Venser vs. Koth\"");
                                                                }
                                                              }
                                                              if (result0 === null) {
                                                                if (input.substr(pos, 31) === "Duel Decks: Knights vs. Dragons") {
                                                                  result0 = "Duel Decks: Knights vs. Dragons";
                                                                  pos += 31;
                                                                } else {
                                                                  result0 = null;
                                                                  if (reportFailures === 0) {
                                                                    matchFailed("\"Duel Decks: Knights vs. Dragons\"");
                                                                  }
                                                                }
                                                                if (result0 === null) {
                                                                  if (input.substr(pos, 29) === "Duel Decks: Elves vs. Goblins") {
                                                                    result0 = "Duel Decks: Elves vs. Goblins";
                                                                    pos += 29;
                                                                  } else {
                                                                    result0 = null;
                                                                    if (reportFailures === 0) {
                                                                      matchFailed("\"Duel Decks: Elves vs. Goblins\"");
                                                                    }
                                                                  }
                                                                  if (result0 === null) {
                                                                    if (input.substr(pos, 28) === "Duel Decks: Jace vs. Chandra") {
                                                                      result0 = "Duel Decks: Jace vs. Chandra";
                                                                      pos += 28;
                                                                    } else {
                                                                      result0 = null;
                                                                      if (reportFailures === 0) {
                                                                        matchFailed("\"Duel Decks: Jace vs. Chandra\"");
                                                                      }
                                                                    }
                                                                    if (result0 === null) {
                                                                      if (input.substr(pos, 30) === "Duel Decks: Garruk vs. Liliana") {
                                                                        result0 = "Duel Decks: Garruk vs. Liliana";
                                                                        pos += 30;
                                                                      } else {
                                                                        result0 = null;
                                                                        if (reportFailures === 0) {
                                                                          matchFailed("\"Duel Decks: Garruk vs. Liliana\"");
                                                                        }
                                                                      }
                                                                      if (result0 === null) {
                                                                        if (input.substr(pos, 38) === "Duel Decks: Phyrexia vs. the Coalition") {
                                                                          result0 = "Duel Decks: Phyrexia vs. the Coalition";
                                                                          pos += 38;
                                                                        } else {
                                                                          result0 = null;
                                                                          if (reportFailures === 0) {
                                                                            matchFailed("\"Duel Decks: Phyrexia vs. the Coalition\"");
                                                                          }
                                                                        }
                                                                        if (result0 === null) {
                                                                          if (input.substr(pos, 32) === "Duel Decks: Elspeth vs. Tezzeret") {
                                                                            result0 = "Duel Decks: Elspeth vs. Tezzeret";
                                                                            pos += 32;
                                                                          } else {
                                                                            result0 = null;
                                                                            if (reportFailures === 0) {
                                                                              matchFailed("\"Duel Decks: Elspeth vs. Tezzeret\"");
                                                                            }
                                                                          }
                                                                          if (result0 === null) {
                                                                            if (input.substr(pos, 33) === "Duel Decks: Ajani vs. Nicol Bolas") {
                                                                              result0 = "Duel Decks: Ajani vs. Nicol Bolas";
                                                                              pos += 33;
                                                                            } else {
                                                                              result0 = null;
                                                                              if (reportFailures === 0) {
                                                                                matchFailed("\"Duel Decks: Ajani vs. Nicol Bolas\"");
                                                                              }
                                                                            }
                                                                            if (result0 === null) {
                                                                              if (input.substr(pos, 29) === "Duel Decks: Izzet vs. Golgari") {
                                                                                result0 = "Duel Decks: Izzet vs. Golgari";
                                                                                pos += 29;
                                                                              } else {
                                                                                result0 = null;
                                                                                if (reportFailures === 0) {
                                                                                  matchFailed("\"Duel Decks: Izzet vs. Golgari\"");
                                                                                }
                                                                              }
                                                                              if (result0 === null) {
                                                                                if (input.substr(pos, 28) === "Duel Decks: Sorin vs. Tibalt") {
                                                                                  result0 = "Duel Decks: Sorin vs. Tibalt";
                                                                                  pos += 28;
                                                                                } else {
                                                                                  result0 = null;
                                                                                  if (reportFailures === 0) {
                                                                                    matchFailed("\"Duel Decks: Sorin vs. Tibalt\"");
                                                                                  }
                                                                                }
                                                                                if (result0 === null) {
                                                                                  if (input.substr(pos, 31) === "Duel Decks: Heroes vs. Monsters") {
                                                                                    result0 = "Duel Decks: Heroes vs. Monsters";
                                                                                    pos += 31;
                                                                                  } else {
                                                                                    result0 = null;
                                                                                    if (reportFailures === 0) {
                                                                                      matchFailed("\"Duel Decks: Heroes vs. Monsters\"");
                                                                                    }
                                                                                  }
                                                                                  if (result0 === null) {
                                                                                    if (input.substr(pos, 23) === "From the Vault: Dragons") {
                                                                                      result0 = "From the Vault: Dragons";
                                                                                      pos += 23;
                                                                                    } else {
                                                                                      result0 = null;
                                                                                      if (reportFailures === 0) {
                                                                                        matchFailed("\"From the Vault: Dragons\"");
                                                                                      }
                                                                                    }
                                                                                    if (result0 === null) {
                                                                                      if (input.substr(pos, 22) === "From the Vault: Exiled") {
                                                                                        result0 = "From the Vault: Exiled";
                                                                                        pos += 22;
                                                                                      } else {
                                                                                        result0 = null;
                                                                                        if (reportFailures === 0) {
                                                                                          matchFailed("\"From the Vault: Exiled\"");
                                                                                        }
                                                                                      }
                                                                                      if (result0 === null) {
                                                                                        if (input.substr(pos, 23) === "From the Vault: Legends") {
                                                                                          result0 = "From the Vault: Legends";
                                                                                          pos += 23;
                                                                                        } else {
                                                                                          result0 = null;
                                                                                          if (reportFailures === 0) {
                                                                                            matchFailed("\"From the Vault: Legends\"");
                                                                                          }
                                                                                        }
                                                                                        if (result0 === null) {
                                                                                          if (input.substr(pos, 22) === "From the Vault: Realms") {
                                                                                            result0 = "From the Vault: Realms";
                                                                                            pos += 22;
                                                                                          } else {
                                                                                            result0 = null;
                                                                                            if (reportFailures === 0) {
                                                                                              matchFailed("\"From the Vault: Realms\"");
                                                                                            }
                                                                                          }
                                                                                          if (result0 === null) {
                                                                                            if (input.substr(pos, 22) === "From the Vault: Relics") {
                                                                                              result0 = "From the Vault: Relics";
                                                                                              pos += 22;
                                                                                            } else {
                                                                                              result0 = null;
                                                                                              if (reportFailures === 0) {
                                                                                                matchFailed("\"From the Vault: Relics\"");
                                                                                              }
                                                                                            }
                                                                                            if (result0 === null) {
                                                                                              if (input.substr(pos, 39) === "Premium Deck Series: Fire and Lightning") {
                                                                                                result0 = "Premium Deck Series: Fire and Lightning";
                                                                                                pos += 39;
                                                                                              } else {
                                                                                                result0 = null;
                                                                                                if (reportFailures === 0) {
                                                                                                  matchFailed("\"Premium Deck Series: Fire and Lightning\"");
                                                                                                }
                                                                                              }
                                                                                              if (result0 === null) {
                                                                                                if (input.substr(pos, 30) === "Premium Deck Series: Graveborn") {
                                                                                                  result0 = "Premium Deck Series: Graveborn";
                                                                                                  pos += 30;
                                                                                                } else {
                                                                                                  result0 = null;
                                                                                                  if (reportFailures === 0) {
                                                                                                    matchFailed("\"Premium Deck Series: Graveborn\"");
                                                                                                  }
                                                                                                }
                                                                                                if (result0 === null) {
                                                                                                  if (input.substr(pos, 28) === "Premium Deck Series: Slivers") {
                                                                                                    result0 = "Premium Deck Series: Slivers";
                                                                                                    pos += 28;
                                                                                                  } else {
                                                                                                    result0 = null;
                                                                                                    if (reportFailures === 0) {
                                                                                                      matchFailed("\"Premium Deck Series: Slivers\"");
                                                                                                    }
                                                                                                  }
                                                                                                  if (result0 === null) {
                                                                                                    if (input.substr(pos, 22) === "Promo set for Gatherer") {
                                                                                                      result0 = "Promo set for Gatherer";
                                                                                                      pos += 22;
                                                                                                    } else {
                                                                                                      result0 = null;
                                                                                                      if (reportFailures === 0) {
                                                                                                        matchFailed("\"Promo set for Gatherer\"");
                                                                                                      }
                                                                                                    }
                                                                                                    if (result0 === null) {
                                                                                                      if (input.substr(pos, 25) === "Time Spiral \"Timeshifted\"") {
                                                                                                        result0 = "Time Spiral \"Timeshifted\"";
                                                                                                        pos += 25;
                                                                                                      } else {
                                                                                                        result0 = null;
                                                                                                        if (reportFailures === 0) {
                                                                                                          matchFailed("\"Time Spiral \\\"Timeshifted\\\"\"");
                                                                                                        }
                                                                                                      }
                                                                                                      if (result0 === null) {
                                                                                                        if (input.substr(pos, 9) === "Archenemy") {
                                                                                                          result0 = "Archenemy";
                                                                                                          pos += 9;
                                                                                                        } else {
                                                                                                          result0 = null;
                                                                                                          if (reportFailures === 0) {
                                                                                                            matchFailed("\"Archenemy\"");
                                                                                                          }
                                                                                                        }
                                                                                                        if (result0 === null) {
                                                                                                          if (input.substr(pos, 23) === "Planechase 2012 Edition") {
                                                                                                            result0 = "Planechase 2012 Edition";
                                                                                                            pos += 23;
                                                                                                          } else {
                                                                                                            result0 = null;
                                                                                                            if (reportFailures === 0) {
                                                                                                              matchFailed("\"Planechase 2012 Edition\"");
                                                                                                            }
                                                                                                          }
                                                                                                          if (result0 === null) {
                                                                                                            if (input.substr(pos, 10) === "Planechase") {
                                                                                                              result0 = "Planechase";
                                                                                                              pos += 10;
                                                                                                            } else {
                                                                                                              result0 = null;
                                                                                                              if (reportFailures === 0) {
                                                                                                                matchFailed("\"Planechase\"");
                                                                                                              }
                                                                                                            }
                                                                                                            if (result0 === null) {
                                                                                                              if (input.substr(pos, 30) === "Magic: The Gathering-Commander") {
                                                                                                                result0 = "Magic: The Gathering-Commander";
                                                                                                                pos += 30;
                                                                                                              } else {
                                                                                                                result0 = null;
                                                                                                                if (reportFailures === 0) {
                                                                                                                  matchFailed("\"Magic: The Gathering-Commander\"");
                                                                                                                }
                                                                                                              }
                                                                                                              if (result0 === null) {
                                                                                                                if (input.substr(pos, 21) === "Battle Royale Box Set") {
                                                                                                                  result0 = "Battle Royale Box Set";
                                                                                                                  pos += 21;
                                                                                                                } else {
                                                                                                                  result0 = null;
                                                                                                                  if (reportFailures === 0) {
                                                                                                                    matchFailed("\"Battle Royale Box Set\"");
                                                                                                                  }
                                                                                                                }
                                                                                                                if (result0 === null) {
                                                                                                                  if (input.substr(pos, 11) === "Anthologies") {
                                                                                                                    result0 = "Anthologies";
                                                                                                                    pos += 11;
                                                                                                                  } else {
                                                                                                                    result0 = null;
                                                                                                                    if (reportFailures === 0) {
                                                                                                                      matchFailed("\"Anthologies\"");
                                                                                                                    }
                                                                                                                  }
                                                                                                                  if (result0 === null) {
                                                                                                                    if (input.substr(pos, 10) === "Chronicles") {
                                                                                                                      result0 = "Chronicles";
                                                                                                                      pos += 10;
                                                                                                                    } else {
                                                                                                                      result0 = null;
                                                                                                                      if (reportFailures === 0) {
                                                                                                                        matchFailed("\"Chronicles\"");
                                                                                                                      }
                                                                                                                    }
                                                                                                                    if (result0 === null) {
                                                                                                                      if (input.substr(pos, 12) === "Starter 2000") {
                                                                                                                        result0 = "Starter 2000";
                                                                                                                        pos += 12;
                                                                                                                      } else {
                                                                                                                        result0 = null;
                                                                                                                        if (reportFailures === 0) {
                                                                                                                          matchFailed("\"Starter 2000\"");
                                                                                                                        }
                                                                                                                      }
                                                                                                                      if (result0 === null) {
                                                                                                                        if (input.substr(pos, 12) === "Starter 1999") {
                                                                                                                          result0 = "Starter 1999";
                                                                                                                          pos += 12;
                                                                                                                        } else {
                                                                                                                          result0 = null;
                                                                                                                          if (reportFailures === 0) {
                                                                                                                            matchFailed("\"Starter 1999\"");
                                                                                                                          }
                                                                                                                        }
                                                                                                                        if (result0 === null) {
                                                                                                                          if (input.substr(pos, 21) === "Portal Three Kingdoms") {
                                                                                                                            result0 = "Portal Three Kingdoms";
                                                                                                                            pos += 21;
                                                                                                                          } else {
                                                                                                                            result0 = null;
                                                                                                                            if (reportFailures === 0) {
                                                                                                                              matchFailed("\"Portal Three Kingdoms\"");
                                                                                                                            }
                                                                                                                          }
                                                                                                                          if (result0 === null) {
                                                                                                                            if (input.substr(pos, 17) === "Portal Second Age") {
                                                                                                                              result0 = "Portal Second Age";
                                                                                                                              pos += 17;
                                                                                                                            } else {
                                                                                                                              result0 = null;
                                                                                                                              if (reportFailures === 0) {
                                                                                                                                matchFailed("\"Portal Second Age\"");
                                                                                                                              }
                                                                                                                            }
                                                                                                                            if (result0 === null) {
                                                                                                                              if (input.substr(pos, 6) === "Portal") {
                                                                                                                                result0 = "Portal";
                                                                                                                                pos += 6;
                                                                                                                              } else {
                                                                                                                                result0 = null;
                                                                                                                                if (reportFailures === 0) {
                                                                                                                                  matchFailed("\"Portal\"");
                                                                                                                                }
                                                                                                                              }
                                                                                                                              if (result0 === null) {
                                                                                                                                if (input.substr(pos, 16) === "Journey into Nyx") {
                                                                                                                                  result0 = "Journey into Nyx";
                                                                                                                                  pos += 16;
                                                                                                                                } else {
                                                                                                                                  result0 = null;
                                                                                                                                  if (reportFailures === 0) {
                                                                                                                                    matchFailed("\"Journey into Nyx\"");
                                                                                                                                  }
                                                                                                                                }
                                                                                                                                if (result0 === null) {
                                                                                                                                  if (input.substr(pos, 16) === "Born of the Gods") {
                                                                                                                                    result0 = "Born of the Gods";
                                                                                                                                    pos += 16;
                                                                                                                                  } else {
                                                                                                                                    result0 = null;
                                                                                                                                    if (reportFailures === 0) {
                                                                                                                                      matchFailed("\"Born of the Gods\"");
                                                                                                                                    }
                                                                                                                                  }
                                                                                                                                  if (result0 === null) {
                                                                                                                                    if (input.substr(pos, 6) === "Theros") {
                                                                                                                                      result0 = "Theros";
                                                                                                                                      pos += 6;
                                                                                                                                    } else {
                                                                                                                                      result0 = null;
                                                                                                                                      if (reportFailures === 0) {
                                                                                                                                        matchFailed("\"Theros\"");
                                                                                                                                      }
                                                                                                                                    }
                                                                                                                                    if (result0 === null) {
                                                                                                                                      if (input.substr(pos, 13) === "Dragon's Maze") {
                                                                                                                                        result0 = "Dragon's Maze";
                                                                                                                                        pos += 13;
                                                                                                                                      } else {
                                                                                                                                        result0 = null;
                                                                                                                                        if (reportFailures === 0) {
                                                                                                                                          matchFailed("\"Dragon's Maze\"");
                                                                                                                                        }
                                                                                                                                      }
                                                                                                                                      if (result0 === null) {
                                                                                                                                        if (input.substr(pos, 9) === "Gatecrash") {
                                                                                                                                          result0 = "Gatecrash";
                                                                                                                                          pos += 9;
                                                                                                                                        } else {
                                                                                                                                          result0 = null;
                                                                                                                                          if (reportFailures === 0) {
                                                                                                                                            matchFailed("\"Gatecrash\"");
                                                                                                                                          }
                                                                                                                                        }
                                                                                                                                        if (result0 === null) {
                                                                                                                                          if (input.substr(pos, 17) === "Return to Ravnica") {
                                                                                                                                            result0 = "Return to Ravnica";
                                                                                                                                            pos += 17;
                                                                                                                                          } else {
                                                                                                                                            result0 = null;
                                                                                                                                            if (reportFailures === 0) {
                                                                                                                                              matchFailed("\"Return to Ravnica\"");
                                                                                                                                            }
                                                                                                                                          }
                                                                                                                                          if (result0 === null) {
                                                                                                                                            if (input.substr(pos, 15) === "Avacyn Restored") {
                                                                                                                                              result0 = "Avacyn Restored";
                                                                                                                                              pos += 15;
                                                                                                                                            } else {
                                                                                                                                              result0 = null;
                                                                                                                                              if (reportFailures === 0) {
                                                                                                                                                matchFailed("\"Avacyn Restored\"");
                                                                                                                                              }
                                                                                                                                            }
                                                                                                                                            if (result0 === null) {
                                                                                                                                              if (input.substr(pos, 14) === "Dark Ascension") {
                                                                                                                                                result0 = "Dark Ascension";
                                                                                                                                                pos += 14;
                                                                                                                                              } else {
                                                                                                                                                result0 = null;
                                                                                                                                                if (reportFailures === 0) {
                                                                                                                                                  matchFailed("\"Dark Ascension\"");
                                                                                                                                                }
                                                                                                                                              }
                                                                                                                                              if (result0 === null) {
                                                                                                                                                if (input.substr(pos, 9) === "Innistrad") {
                                                                                                                                                  result0 = "Innistrad";
                                                                                                                                                  pos += 9;
                                                                                                                                                } else {
                                                                                                                                                  result0 = null;
                                                                                                                                                  if (reportFailures === 0) {
                                                                                                                                                    matchFailed("\"Innistrad\"");
                                                                                                                                                  }
                                                                                                                                                }
                                                                                                                                                if (result0 === null) {
                                                                                                                                                  if (input.substr(pos, 12) === "New Phyrexia") {
                                                                                                                                                    result0 = "New Phyrexia";
                                                                                                                                                    pos += 12;
                                                                                                                                                  } else {
                                                                                                                                                    result0 = null;
                                                                                                                                                    if (reportFailures === 0) {
                                                                                                                                                      matchFailed("\"New Phyrexia\"");
                                                                                                                                                    }
                                                                                                                                                  }
                                                                                                                                                  if (result0 === null) {
                                                                                                                                                    if (input.substr(pos, 17) === "Mirrodin Besieged") {
                                                                                                                                                      result0 = "Mirrodin Besieged";
                                                                                                                                                      pos += 17;
                                                                                                                                                    } else {
                                                                                                                                                      result0 = null;
                                                                                                                                                      if (reportFailures === 0) {
                                                                                                                                                        matchFailed("\"Mirrodin Besieged\"");
                                                                                                                                                      }
                                                                                                                                                    }
                                                                                                                                                    if (result0 === null) {
                                                                                                                                                      if (input.substr(pos, 17) === "Scars of Mirrodin") {
                                                                                                                                                        result0 = "Scars of Mirrodin";
                                                                                                                                                        pos += 17;
                                                                                                                                                      } else {
                                                                                                                                                        result0 = null;
                                                                                                                                                        if (reportFailures === 0) {
                                                                                                                                                          matchFailed("\"Scars of Mirrodin\"");
                                                                                                                                                        }
                                                                                                                                                      }
                                                                                                                                                      if (result0 === null) {
                                                                                                                                                        if (input.substr(pos, 19) === "Rise of the Eldrazi") {
                                                                                                                                                          result0 = "Rise of the Eldrazi";
                                                                                                                                                          pos += 19;
                                                                                                                                                        } else {
                                                                                                                                                          result0 = null;
                                                                                                                                                          if (reportFailures === 0) {
                                                                                                                                                            matchFailed("\"Rise of the Eldrazi\"");
                                                                                                                                                          }
                                                                                                                                                        }
                                                                                                                                                        if (result0 === null) {
                                                                                                                                                          if (input.substr(pos, 9) === "Worldwake") {
                                                                                                                                                            result0 = "Worldwake";
                                                                                                                                                            pos += 9;
                                                                                                                                                          } else {
                                                                                                                                                            result0 = null;
                                                                                                                                                            if (reportFailures === 0) {
                                                                                                                                                              matchFailed("\"Worldwake\"");
                                                                                                                                                            }
                                                                                                                                                          }
                                                                                                                                                          if (result0 === null) {
                                                                                                                                                            if (input.substr(pos, 8) === "Zendikar") {
                                                                                                                                                              result0 = "Zendikar";
                                                                                                                                                              pos += 8;
                                                                                                                                                            } else {
                                                                                                                                                              result0 = null;
                                                                                                                                                              if (reportFailures === 0) {
                                                                                                                                                                matchFailed("\"Zendikar\"");
                                                                                                                                                              }
                                                                                                                                                            }
                                                                                                                                                            if (result0 === null) {
                                                                                                                                                              if (input.substr(pos, 12) === "Alara Reborn") {
                                                                                                                                                                result0 = "Alara Reborn";
                                                                                                                                                                pos += 12;
                                                                                                                                                              } else {
                                                                                                                                                                result0 = null;
                                                                                                                                                                if (reportFailures === 0) {
                                                                                                                                                                  matchFailed("\"Alara Reborn\"");
                                                                                                                                                                }
                                                                                                                                                              }
                                                                                                                                                              if (result0 === null) {
                                                                                                                                                                if (input.substr(pos, 7) === "Conflux") {
                                                                                                                                                                  result0 = "Conflux";
                                                                                                                                                                  pos += 7;
                                                                                                                                                                } else {
                                                                                                                                                                  result0 = null;
                                                                                                                                                                  if (reportFailures === 0) {
                                                                                                                                                                    matchFailed("\"Conflux\"");
                                                                                                                                                                  }
                                                                                                                                                                }
                                                                                                                                                                if (result0 === null) {
                                                                                                                                                                  if (input.substr(pos, 15) === "Shards of Alara") {
                                                                                                                                                                    result0 = "Shards of Alara";
                                                                                                                                                                    pos += 15;
                                                                                                                                                                  } else {
                                                                                                                                                                    result0 = null;
                                                                                                                                                                    if (reportFailures === 0) {
                                                                                                                                                                      matchFailed("\"Shards of Alara\"");
                                                                                                                                                                    }
                                                                                                                                                                  }
                                                                                                                                                                  if (result0 === null) {
                                                                                                                                                                    if (input.substr(pos, 8) === "Eventide") {
                                                                                                                                                                      result0 = "Eventide";
                                                                                                                                                                      pos += 8;
                                                                                                                                                                    } else {
                                                                                                                                                                      result0 = null;
                                                                                                                                                                      if (reportFailures === 0) {
                                                                                                                                                                        matchFailed("\"Eventide\"");
                                                                                                                                                                      }
                                                                                                                                                                    }
                                                                                                                                                                    if (result0 === null) {
                                                                                                                                                                      if (input.substr(pos, 10) === "Shadowmoor") {
                                                                                                                                                                        result0 = "Shadowmoor";
                                                                                                                                                                        pos += 10;
                                                                                                                                                                      } else {
                                                                                                                                                                        result0 = null;
                                                                                                                                                                        if (reportFailures === 0) {
                                                                                                                                                                          matchFailed("\"Shadowmoor\"");
                                                                                                                                                                        }
                                                                                                                                                                      }
                                                                                                                                                                      if (result0 === null) {
                                                                                                                                                                        if (input.substr(pos, 11) === "Morningtide") {
                                                                                                                                                                          result0 = "Morningtide";
                                                                                                                                                                          pos += 11;
                                                                                                                                                                        } else {
                                                                                                                                                                          result0 = null;
                                                                                                                                                                          if (reportFailures === 0) {
                                                                                                                                                                            matchFailed("\"Morningtide\"");
                                                                                                                                                                          }
                                                                                                                                                                        }
                                                                                                                                                                        if (result0 === null) {
                                                                                                                                                                          if (input.substr(pos, 6) === "Lorwyn") {
                                                                                                                                                                            result0 = "Lorwyn";
                                                                                                                                                                            pos += 6;
                                                                                                                                                                          } else {
                                                                                                                                                                            result0 = null;
                                                                                                                                                                            if (reportFailures === 0) {
                                                                                                                                                                              matchFailed("\"Lorwyn\"");
                                                                                                                                                                            }
                                                                                                                                                                          }
                                                                                                                                                                          if (result0 === null) {
                                                                                                                                                                            if (input.substr(pos, 12) === "Future Sight") {
                                                                                                                                                                              result0 = "Future Sight";
                                                                                                                                                                              pos += 12;
                                                                                                                                                                            } else {
                                                                                                                                                                              result0 = null;
                                                                                                                                                                              if (reportFailures === 0) {
                                                                                                                                                                                matchFailed("\"Future Sight\"");
                                                                                                                                                                              }
                                                                                                                                                                            }
                                                                                                                                                                            if (result0 === null) {
                                                                                                                                                                              if (input.substr(pos, 12) === "Planar Chaos") {
                                                                                                                                                                                result0 = "Planar Chaos";
                                                                                                                                                                                pos += 12;
                                                                                                                                                                              } else {
                                                                                                                                                                                result0 = null;
                                                                                                                                                                                if (reportFailures === 0) {
                                                                                                                                                                                  matchFailed("\"Planar Chaos\"");
                                                                                                                                                                                }
                                                                                                                                                                              }
                                                                                                                                                                              if (result0 === null) {
                                                                                                                                                                                if (input.substr(pos, 11) === "Time Spiral") {
                                                                                                                                                                                  result0 = "Time Spiral";
                                                                                                                                                                                  pos += 11;
                                                                                                                                                                                } else {
                                                                                                                                                                                  result0 = null;
                                                                                                                                                                                  if (reportFailures === 0) {
                                                                                                                                                                                    matchFailed("\"Time Spiral\"");
                                                                                                                                                                                  }
                                                                                                                                                                                }
                                                                                                                                                                                if (result0 === null) {
                                                                                                                                                                                  if (input.substr(pos, 10) === "Dissension") {
                                                                                                                                                                                    result0 = "Dissension";
                                                                                                                                                                                    pos += 10;
                                                                                                                                                                                  } else {
                                                                                                                                                                                    result0 = null;
                                                                                                                                                                                    if (reportFailures === 0) {
                                                                                                                                                                                      matchFailed("\"Dissension\"");
                                                                                                                                                                                    }
                                                                                                                                                                                  }
                                                                                                                                                                                  if (result0 === null) {
                                                                                                                                                                                    if (input.substr(pos, 9) === "Guildpact") {
                                                                                                                                                                                      result0 = "Guildpact";
                                                                                                                                                                                      pos += 9;
                                                                                                                                                                                    } else {
                                                                                                                                                                                      result0 = null;
                                                                                                                                                                                      if (reportFailures === 0) {
                                                                                                                                                                                        matchFailed("\"Guildpact\"");
                                                                                                                                                                                      }
                                                                                                                                                                                    }
                                                                                                                                                                                    if (result0 === null) {
                                                                                                                                                                                      if (input.substr(pos, 23) === "Ravnica: City of Guilds") {
                                                                                                                                                                                        result0 = "Ravnica: City of Guilds";
                                                                                                                                                                                        pos += 23;
                                                                                                                                                                                      } else {
                                                                                                                                                                                        result0 = null;
                                                                                                                                                                                        if (reportFailures === 0) {
                                                                                                                                                                                          matchFailed("\"Ravnica: City of Guilds\"");
                                                                                                                                                                                        }
                                                                                                                                                                                      }
                                                                                                                                                                                      if (result0 === null) {
                                                                                                                                                                                        if (input.substr(pos, 19) === "Saviors of Kamigawa") {
                                                                                                                                                                                          result0 = "Saviors of Kamigawa";
                                                                                                                                                                                          pos += 19;
                                                                                                                                                                                        } else {
                                                                                                                                                                                          result0 = null;
                                                                                                                                                                                          if (reportFailures === 0) {
                                                                                                                                                                                            matchFailed("\"Saviors of Kamigawa\"");
                                                                                                                                                                                          }
                                                                                                                                                                                        }
                                                                                                                                                                                        if (result0 === null) {
                                                                                                                                                                                          if (input.substr(pos, 21) === "Betrayers of Kamigawa") {
                                                                                                                                                                                            result0 = "Betrayers of Kamigawa";
                                                                                                                                                                                            pos += 21;
                                                                                                                                                                                          } else {
                                                                                                                                                                                            result0 = null;
                                                                                                                                                                                            if (reportFailures === 0) {
                                                                                                                                                                                              matchFailed("\"Betrayers of Kamigawa\"");
                                                                                                                                                                                            }
                                                                                                                                                                                          }
                                                                                                                                                                                          if (result0 === null) {
                                                                                                                                                                                            if (input.substr(pos, 21) === "Champions of Kamigawa") {
                                                                                                                                                                                              result0 = "Champions of Kamigawa";
                                                                                                                                                                                              pos += 21;
                                                                                                                                                                                            } else {
                                                                                                                                                                                              result0 = null;
                                                                                                                                                                                              if (reportFailures === 0) {
                                                                                                                                                                                                matchFailed("\"Champions of Kamigawa\"");
                                                                                                                                                                                              }
                                                                                                                                                                                            }
                                                                                                                                                                                            if (result0 === null) {
                                                                                                                                                                                              if (input.substr(pos, 10) === "Fifth Dawn") {
                                                                                                                                                                                                result0 = "Fifth Dawn";
                                                                                                                                                                                                pos += 10;
                                                                                                                                                                                              } else {
                                                                                                                                                                                                result0 = null;
                                                                                                                                                                                                if (reportFailures === 0) {
                                                                                                                                                                                                  matchFailed("\"Fifth Dawn\"");
                                                                                                                                                                                                }
                                                                                                                                                                                              }
                                                                                                                                                                                              if (result0 === null) {
                                                                                                                                                                                                if (input.substr(pos, 9) === "Darksteel") {
                                                                                                                                                                                                  result0 = "Darksteel";
                                                                                                                                                                                                  pos += 9;
                                                                                                                                                                                                } else {
                                                                                                                                                                                                  result0 = null;
                                                                                                                                                                                                  if (reportFailures === 0) {
                                                                                                                                                                                                    matchFailed("\"Darksteel\"");
                                                                                                                                                                                                  }
                                                                                                                                                                                                }
                                                                                                                                                                                                if (result0 === null) {
                                                                                                                                                                                                  if (input.substr(pos, 8) === "Mirrodin") {
                                                                                                                                                                                                    result0 = "Mirrodin";
                                                                                                                                                                                                    pos += 8;
                                                                                                                                                                                                  } else {
                                                                                                                                                                                                    result0 = null;
                                                                                                                                                                                                    if (reportFailures === 0) {
                                                                                                                                                                                                      matchFailed("\"Mirrodin\"");
                                                                                                                                                                                                    }
                                                                                                                                                                                                  }
                                                                                                                                                                                                  if (result0 === null) {
                                                                                                                                                                                                    if (input.substr(pos, 7) === "Scourge") {
                                                                                                                                                                                                      result0 = "Scourge";
                                                                                                                                                                                                      pos += 7;
                                                                                                                                                                                                    } else {
                                                                                                                                                                                                      result0 = null;
                                                                                                                                                                                                      if (reportFailures === 0) {
                                                                                                                                                                                                        matchFailed("\"Scourge\"");
                                                                                                                                                                                                      }
                                                                                                                                                                                                    }
                                                                                                                                                                                                    if (result0 === null) {
                                                                                                                                                                                                      if (input.substr(pos, 7) === "Legions") {
                                                                                                                                                                                                        result0 = "Legions";
                                                                                                                                                                                                        pos += 7;
                                                                                                                                                                                                      } else {
                                                                                                                                                                                                        result0 = null;
                                                                                                                                                                                                        if (reportFailures === 0) {
                                                                                                                                                                                                          matchFailed("\"Legions\"");
                                                                                                                                                                                                        }
                                                                                                                                                                                                      }
                                                                                                                                                                                                      if (result0 === null) {
                                                                                                                                                                                                        if (input.substr(pos, 9) === "Onslaught") {
                                                                                                                                                                                                          result0 = "Onslaught";
                                                                                                                                                                                                          pos += 9;
                                                                                                                                                                                                        } else {
                                                                                                                                                                                                          result0 = null;
                                                                                                                                                                                                          if (reportFailures === 0) {
                                                                                                                                                                                                            matchFailed("\"Onslaught\"");
                                                                                                                                                                                                          }
                                                                                                                                                                                                        }
                                                                                                                                                                                                        if (result0 === null) {
                                                                                                                                                                                                          if (input.substr(pos, 8) === "Judgment") {
                                                                                                                                                                                                            result0 = "Judgment";
                                                                                                                                                                                                            pos += 8;
                                                                                                                                                                                                          } else {
                                                                                                                                                                                                            result0 = null;
                                                                                                                                                                                                            if (reportFailures === 0) {
                                                                                                                                                                                                              matchFailed("\"Judgment\"");
                                                                                                                                                                                                            }
                                                                                                                                                                                                          }
                                                                                                                                                                                                          if (result0 === null) {
                                                                                                                                                                                                            if (input.substr(pos, 7) === "Torment") {
                                                                                                                                                                                                              result0 = "Torment";
                                                                                                                                                                                                              pos += 7;
                                                                                                                                                                                                            } else {
                                                                                                                                                                                                              result0 = null;
                                                                                                                                                                                                              if (reportFailures === 0) {
                                                                                                                                                                                                                matchFailed("\"Torment\"");
                                                                                                                                                                                                              }
                                                                                                                                                                                                            }
                                                                                                                                                                                                            if (result0 === null) {
                                                                                                                                                                                                              if (input.substr(pos, 7) === "Odyssey") {
                                                                                                                                                                                                                result0 = "Odyssey";
                                                                                                                                                                                                                pos += 7;
                                                                                                                                                                                                              } else {
                                                                                                                                                                                                                result0 = null;
                                                                                                                                                                                                                if (reportFailures === 0) {
                                                                                                                                                                                                                  matchFailed("\"Odyssey\"");
                                                                                                                                                                                                                }
                                                                                                                                                                                                              }
                                                                                                                                                                                                              if (result0 === null) {
                                                                                                                                                                                                                if (input.substr(pos, 10) === "Apocalypse") {
                                                                                                                                                                                                                  result0 = "Apocalypse";
                                                                                                                                                                                                                  pos += 10;
                                                                                                                                                                                                                } else {
                                                                                                                                                                                                                  result0 = null;
                                                                                                                                                                                                                  if (reportFailures === 0) {
                                                                                                                                                                                                                    matchFailed("\"Apocalypse\"");
                                                                                                                                                                                                                  }
                                                                                                                                                                                                                }
                                                                                                                                                                                                                if (result0 === null) {
                                                                                                                                                                                                                  if (input.substr(pos, 10) === "Planeshift") {
                                                                                                                                                                                                                    result0 = "Planeshift";
                                                                                                                                                                                                                    pos += 10;
                                                                                                                                                                                                                  } else {
                                                                                                                                                                                                                    result0 = null;
                                                                                                                                                                                                                    if (reportFailures === 0) {
                                                                                                                                                                                                                      matchFailed("\"Planeshift\"");
                                                                                                                                                                                                                    }
                                                                                                                                                                                                                  }
                                                                                                                                                                                                                  if (result0 === null) {
                                                                                                                                                                                                                    if (input.substr(pos, 8) === "Invasion") {
                                                                                                                                                                                                                      result0 = "Invasion";
                                                                                                                                                                                                                      pos += 8;
                                                                                                                                                                                                                    } else {
                                                                                                                                                                                                                      result0 = null;
                                                                                                                                                                                                                      if (reportFailures === 0) {
                                                                                                                                                                                                                        matchFailed("\"Invasion\"");
                                                                                                                                                                                                                      }
                                                                                                                                                                                                                    }
                                                                                                                                                                                                                    if (result0 === null) {
                                                                                                                                                                                                                      if (input.substr(pos, 8) === "Prophecy") {
                                                                                                                                                                                                                        result0 = "Prophecy";
                                                                                                                                                                                                                        pos += 8;
                                                                                                                                                                                                                      } else {
                                                                                                                                                                                                                        result0 = null;
                                                                                                                                                                                                                        if (reportFailures === 0) {
                                                                                                                                                                                                                          matchFailed("\"Prophecy\"");
                                                                                                                                                                                                                        }
                                                                                                                                                                                                                      }
                                                                                                                                                                                                                      if (result0 === null) {
                                                                                                                                                                                                                        if (input.substr(pos, 7) === "Nemesis") {
                                                                                                                                                                                                                          result0 = "Nemesis";
                                                                                                                                                                                                                          pos += 7;
                                                                                                                                                                                                                        } else {
                                                                                                                                                                                                                          result0 = null;
                                                                                                                                                                                                                          if (reportFailures === 0) {
                                                                                                                                                                                                                            matchFailed("\"Nemesis\"");
                                                                                                                                                                                                                          }
                                                                                                                                                                                                                        }
                                                                                                                                                                                                                        if (result0 === null) {
                                                                                                                                                                                                                          if (input.substr(pos, 17) === "Mercadian Masques") {
                                                                                                                                                                                                                            result0 = "Mercadian Masques";
                                                                                                                                                                                                                            pos += 17;
                                                                                                                                                                                                                          } else {
                                                                                                                                                                                                                            result0 = null;
                                                                                                                                                                                                                            if (reportFailures === 0) {
                                                                                                                                                                                                                              matchFailed("\"Mercadian Masques\"");
                                                                                                                                                                                                                            }
                                                                                                                                                                                                                          }
                                                                                                                                                                                                                          if (result0 === null) {
                                                                                                                                                                                                                            if (input.substr(pos, 14) === "Urza's Destiny") {
                                                                                                                                                                                                                              result0 = "Urza's Destiny";
                                                                                                                                                                                                                              pos += 14;
                                                                                                                                                                                                                            } else {
                                                                                                                                                                                                                              result0 = null;
                                                                                                                                                                                                                              if (reportFailures === 0) {
                                                                                                                                                                                                                                matchFailed("\"Urza's Destiny\"");
                                                                                                                                                                                                                              }
                                                                                                                                                                                                                            }
                                                                                                                                                                                                                            if (result0 === null) {
                                                                                                                                                                                                                              if (input.substr(pos, 13) === "Urza's Legacy") {
                                                                                                                                                                                                                                result0 = "Urza's Legacy";
                                                                                                                                                                                                                                pos += 13;
                                                                                                                                                                                                                              } else {
                                                                                                                                                                                                                                result0 = null;
                                                                                                                                                                                                                                if (reportFailures === 0) {
                                                                                                                                                                                                                                  matchFailed("\"Urza's Legacy\"");
                                                                                                                                                                                                                                }
                                                                                                                                                                                                                              }
                                                                                                                                                                                                                              if (result0 === null) {
                                                                                                                                                                                                                                if (input.substr(pos, 11) === "Urza's Saga") {
                                                                                                                                                                                                                                  result0 = "Urza's Saga";
                                                                                                                                                                                                                                  pos += 11;
                                                                                                                                                                                                                                } else {
                                                                                                                                                                                                                                  result0 = null;
                                                                                                                                                                                                                                  if (reportFailures === 0) {
                                                                                                                                                                                                                                    matchFailed("\"Urza's Saga\"");
                                                                                                                                                                                                                                  }
                                                                                                                                                                                                                                }
                                                                                                                                                                                                                                if (result0 === null) {
                                                                                                                                                                                                                                  if (input.substr(pos, 6) === "Exodus") {
                                                                                                                                                                                                                                    result0 = "Exodus";
                                                                                                                                                                                                                                    pos += 6;
                                                                                                                                                                                                                                  } else {
                                                                                                                                                                                                                                    result0 = null;
                                                                                                                                                                                                                                    if (reportFailures === 0) {
                                                                                                                                                                                                                                      matchFailed("\"Exodus\"");
                                                                                                                                                                                                                                    }
                                                                                                                                                                                                                                  }
                                                                                                                                                                                                                                  if (result0 === null) {
                                                                                                                                                                                                                                    if (input.substr(pos, 10) === "Stronghold") {
                                                                                                                                                                                                                                      result0 = "Stronghold";
                                                                                                                                                                                                                                      pos += 10;
                                                                                                                                                                                                                                    } else {
                                                                                                                                                                                                                                      result0 = null;
                                                                                                                                                                                                                                      if (reportFailures === 0) {
                                                                                                                                                                                                                                        matchFailed("\"Stronghold\"");
                                                                                                                                                                                                                                      }
                                                                                                                                                                                                                                    }
                                                                                                                                                                                                                                    if (result0 === null) {
                                                                                                                                                                                                                                      if (input.substr(pos, 7) === "Tempest") {
                                                                                                                                                                                                                                        result0 = "Tempest";
                                                                                                                                                                                                                                        pos += 7;
                                                                                                                                                                                                                                      } else {
                                                                                                                                                                                                                                        result0 = null;
                                                                                                                                                                                                                                        if (reportFailures === 0) {
                                                                                                                                                                                                                                          matchFailed("\"Tempest\"");
                                                                                                                                                                                                                                        }
                                                                                                                                                                                                                                      }
                                                                                                                                                                                                                                      if (result0 === null) {
                                                                                                                                                                                                                                        if (input.substr(pos, 12) === "Weatherlight") {
                                                                                                                                                                                                                                          result0 = "Weatherlight";
                                                                                                                                                                                                                                          pos += 12;
                                                                                                                                                                                                                                        } else {
                                                                                                                                                                                                                                          result0 = null;
                                                                                                                                                                                                                                          if (reportFailures === 0) {
                                                                                                                                                                                                                                            matchFailed("\"Weatherlight\"");
                                                                                                                                                                                                                                          }
                                                                                                                                                                                                                                        }
                                                                                                                                                                                                                                        if (result0 === null) {
                                                                                                                                                                                                                                          if (input.substr(pos, 7) === "Visions") {
                                                                                                                                                                                                                                            result0 = "Visions";
                                                                                                                                                                                                                                            pos += 7;
                                                                                                                                                                                                                                          } else {
                                                                                                                                                                                                                                            result0 = null;
                                                                                                                                                                                                                                            if (reportFailures === 0) {
                                                                                                                                                                                                                                              matchFailed("\"Visions\"");
                                                                                                                                                                                                                                            }
                                                                                                                                                                                                                                          }
                                                                                                                                                                                                                                          if (result0 === null) {
                                                                                                                                                                                                                                            if (input.substr(pos, 6) === "Mirage") {
                                                                                                                                                                                                                                              result0 = "Mirage";
                                                                                                                                                                                                                                              pos += 6;
                                                                                                                                                                                                                                            } else {
                                                                                                                                                                                                                                              result0 = null;
                                                                                                                                                                                                                                              if (reportFailures === 0) {
                                                                                                                                                                                                                                                matchFailed("\"Mirage\"");
                                                                                                                                                                                                                                              }
                                                                                                                                                                                                                                            }
                                                                                                                                                                                                                                            if (result0 === null) {
                                                                                                                                                                                                                                              if (input.substr(pos, 8) === "Coldsnap") {
                                                                                                                                                                                                                                                result0 = "Coldsnap";
                                                                                                                                                                                                                                                pos += 8;
                                                                                                                                                                                                                                              } else {
                                                                                                                                                                                                                                                result0 = null;
                                                                                                                                                                                                                                                if (reportFailures === 0) {
                                                                                                                                                                                                                                                  matchFailed("\"Coldsnap\"");
                                                                                                                                                                                                                                                }
                                                                                                                                                                                                                                              }
                                                                                                                                                                                                                                              if (result0 === null) {
                                                                                                                                                                                                                                                if (input.substr(pos, 9) === "Alliances") {
                                                                                                                                                                                                                                                  result0 = "Alliances";
                                                                                                                                                                                                                                                  pos += 9;
                                                                                                                                                                                                                                                } else {
                                                                                                                                                                                                                                                  result0 = null;
                                                                                                                                                                                                                                                  if (reportFailures === 0) {
                                                                                                                                                                                                                                                    matchFailed("\"Alliances\"");
                                                                                                                                                                                                                                                  }
                                                                                                                                                                                                                                                }
                                                                                                                                                                                                                                                if (result0 === null) {
                                                                                                                                                                                                                                                  if (input.substr(pos, 9) === "Homelands") {
                                                                                                                                                                                                                                                    result0 = "Homelands";
                                                                                                                                                                                                                                                    pos += 9;
                                                                                                                                                                                                                                                  } else {
                                                                                                                                                                                                                                                    result0 = null;
                                                                                                                                                                                                                                                    if (reportFailures === 0) {
                                                                                                                                                                                                                                                      matchFailed("\"Homelands\"");
                                                                                                                                                                                                                                                    }
                                                                                                                                                                                                                                                  }
                                                                                                                                                                                                                                                  if (result0 === null) {
                                                                                                                                                                                                                                                    if (input.substr(pos, 7) === "Ice Age") {
                                                                                                                                                                                                                                                      result0 = "Ice Age";
                                                                                                                                                                                                                                                      pos += 7;
                                                                                                                                                                                                                                                    } else {
                                                                                                                                                                                                                                                      result0 = null;
                                                                                                                                                                                                                                                      if (reportFailures === 0) {
                                                                                                                                                                                                                                                        matchFailed("\"Ice Age\"");
                                                                                                                                                                                                                                                      }
                                                                                                                                                                                                                                                    }
                                                                                                                                                                                                                                                    if (result0 === null) {
                                                                                                                                                                                                                                                      if (input.substr(pos, 14) === "Fallen Empires") {
                                                                                                                                                                                                                                                        result0 = "Fallen Empires";
                                                                                                                                                                                                                                                        pos += 14;
                                                                                                                                                                                                                                                      } else {
                                                                                                                                                                                                                                                        result0 = null;
                                                                                                                                                                                                                                                        if (reportFailures === 0) {
                                                                                                                                                                                                                                                          matchFailed("\"Fallen Empires\"");
                                                                                                                                                                                                                                                        }
                                                                                                                                                                                                                                                      }
                                                                                                                                                                                                                                                      if (result0 === null) {
                                                                                                                                                                                                                                                        if (input.substr(pos, 8) === "The Dark") {
                                                                                                                                                                                                                                                          result0 = "The Dark";
                                                                                                                                                                                                                                                          pos += 8;
                                                                                                                                                                                                                                                        } else {
                                                                                                                                                                                                                                                          result0 = null;
                                                                                                                                                                                                                                                          if (reportFailures === 0) {
                                                                                                                                                                                                                                                            matchFailed("\"The Dark\"");
                                                                                                                                                                                                                                                          }
                                                                                                                                                                                                                                                        }
                                                                                                                                                                                                                                                        if (result0 === null) {
                                                                                                                                                                                                                                                          if (input.substr(pos, 7) === "Legends") {
                                                                                                                                                                                                                                                            result0 = "Legends";
                                                                                                                                                                                                                                                            pos += 7;
                                                                                                                                                                                                                                                          } else {
                                                                                                                                                                                                                                                            result0 = null;
                                                                                                                                                                                                                                                            if (reportFailures === 0) {
                                                                                                                                                                                                                                                              matchFailed("\"Legends\"");
                                                                                                                                                                                                                                                            }
                                                                                                                                                                                                                                                          }
                                                                                                                                                                                                                                                          if (result0 === null) {
                                                                                                                                                                                                                                                            if (input.substr(pos, 11) === "Antiquities") {
                                                                                                                                                                                                                                                              result0 = "Antiquities";
                                                                                                                                                                                                                                                              pos += 11;
                                                                                                                                                                                                                                                            } else {
                                                                                                                                                                                                                                                              result0 = null;
                                                                                                                                                                                                                                                              if (reportFailures === 0) {
                                                                                                                                                                                                                                                                matchFailed("\"Antiquities\"");
                                                                                                                                                                                                                                                              }
                                                                                                                                                                                                                                                            }
                                                                                                                                                                                                                                                            if (result0 === null) {
                                                                                                                                                                                                                                                              if (input.substr(pos, 14) === "Arabian Nights") {
                                                                                                                                                                                                                                                                result0 = "Arabian Nights";
                                                                                                                                                                                                                                                                pos += 14;
                                                                                                                                                                                                                                                              } else {
                                                                                                                                                                                                                                                                result0 = null;
                                                                                                                                                                                                                                                                if (reportFailures === 0) {
                                                                                                                                                                                                                                                                  matchFailed("\"Arabian Nights\"");
                                                                                                                                                                                                                                                                }
                                                                                                                                                                                                                                                              }
                                                                                                                                                                                                                                                            }
                                                                                                                                                                                                                                                          }
                                                                                                                                                                                                                                                        }
                                                                                                                                                                                                                                                      }
                                                                                                                                                                                                                                                    }
                                                                                                                                                                                                                                                  }
                                                                                                                                                                                                                                                }
                                                                                                                                                                                                                                              }
                                                                                                                                                                                                                                            }
                                                                                                                                                                                                                                          }
                                                                                                                                                                                                                                        }
                                                                                                                                                                                                                                      }
                                                                                                                                                                                                                                    }
                                                                                                                                                                                                                                  }
                                                                                                                                                                                                                                }
                                                                                                                                                                                                                              }
                                                                                                                                                                                                                            }
                                                                                                                                                                                                                          }
                                                                                                                                                                                                                        }
                                                                                                                                                                                                                      }
                                                                                                                                                                                                                    }
                                                                                                                                                                                                                  }
                                                                                                                                                                                                                }
                                                                                                                                                                                                              }
                                                                                                                                                                                                            }
                                                                                                                                                                                                          }
                                                                                                                                                                                                        }
                                                                                                                                                                                                      }
                                                                                                                                                                                                    }
                                                                                                                                                                                                  }
                                                                                                                                                                                                }
                                                                                                                                                                                              }
                                                                                                                                                                                            }
                                                                                                                                                                                          }
                                                                                                                                                                                        }
                                                                                                                                                                                      }
                                                                                                                                                                                    }
                                                                                                                                                                                  }
                                                                                                                                                                                }
                                                                                                                                                                              }
                                                                                                                                                                            }
                                                                                                                                                                          }
                                                                                                                                                                        }
                                                                                                                                                                      }
                                                                                                                                                                    }
                                                                                                                                                                  }
                                                                                                                                                                }
                                                                                                                                                              }
                                                                                                                                                            }
                                                                                                                                                          }
                                                                                                                                                        }
                                                                                                                                                      }
                                                                                                                                                    }
                                                                                                                                                  }
                                                                                                                                                }
                                                                                                                                              }
                                                                                                                                            }
                                                                                                                                          }
                                                                                                                                        }
                                                                                                                                      }
                                                                                                                                    }
                                                                                                                                  }
                                                                                                                                }
                                                                                                                              }
                                                                                                                            }
                                                                                                                          }
                                                                                                                        }
                                                                                                                      }
                                                                                                                    }
                                                                                                                  }
                                                                                                                }
                                                                                                              }
                                                                                                            }
                                                                                                          }
                                                                                                        }
                                                                                                      }
                                                                                                    }
                                                                                                  }
                                                                                                }
                                                                                              }
                                                                                            }
                                                                                          }
                                                                                        }
                                                                                      }
                                                                                    }
                                                                                  }
                                                                                }
                                                                              }
                                                                            }
                                                                          }
                                                                        }
                                                                      }
                                                                    }
                                                                  }
                                                                }
                                                              }
                                                            }
                                                          }
                                                        }
                                                      }
                                                    }
                                                  }
                                                }
                                              }
                                            }
                                          }
                                        }
                                      }
                                    }
                                  }
                                }
                              }
                            }
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
        return result0;
      }
      
      function parse_rarity() {
        var result0;
        
        if (input.substr(pos, 10) === "Basic Land") {
          result0 = "Basic Land";
          pos += 10;
        } else {
          result0 = null;
          if (reportFailures === 0) {
            matchFailed("\"Basic Land\"");
          }
        }
        if (result0 === null) {
          if (input.substr(pos, 6) === "Common") {
            result0 = "Common";
            pos += 6;
          } else {
            result0 = null;
            if (reportFailures === 0) {
              matchFailed("\"Common\"");
            }
          }
          if (result0 === null) {
            if (input.substr(pos, 8) === "Uncommon") {
              result0 = "Uncommon";
              pos += 8;
            } else {
              result0 = null;
              if (reportFailures === 0) {
                matchFailed("\"Uncommon\"");
              }
            }
            if (result0 === null) {
              if (input.substr(pos, 4) === "Rare") {
                result0 = "Rare";
                pos += 4;
              } else {
                result0 = null;
                if (reportFailures === 0) {
                  matchFailed("\"Rare\"");
                }
              }
              if (result0 === null) {
                if (input.substr(pos, 11) === "Mythic Rare") {
                  result0 = "Mythic Rare";
                  pos += 11;
                } else {
                  result0 = null;
                  if (reportFailures === 0) {
                    matchFailed("\"Mythic Rare\"");
                  }
                }
                if (result0 === null) {
                  if (input.substr(pos, 4) === "Land") {
                    result0 = "Land";
                    pos += 4;
                  } else {
                    result0 = null;
                    if (reportFailures === 0) {
                      matchFailed("\"Land\"");
                    }
                  }
                  if (result0 === null) {
                    if (input.substr(pos, 7) === "Special") {
                      result0 = "Special";
                      pos += 7;
                    } else {
                      result0 = null;
                      if (reportFailures === 0) {
                        matchFailed("\"Special\"");
                      }
                    }
                    if (result0 === null) {
                      if (input.substr(pos, 5) === "Promo") {
                        result0 = "Promo";
                        pos += 5;
                      } else {
                        result0 = null;
                        if (reportFailures === 0) {
                          matchFailed("\"Promo\"");
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
        return result0;
      }
      
      function parse_rules() {
        var result0, result1, result2, result3, result4, result5;
        var pos0, pos1, pos2, pos3;
        
        pos0 = pos;
        pos1 = pos;
        if (input.substr(pos, 10) === "Rules Text") {
          result0 = "Rules Text";
          pos += 10;
        } else {
          result0 = null;
          if (reportFailures === 0) {
            matchFailed("\"Rules Text\"");
          }
        }
        if (result0 !== null) {
          result1 = parse_colon();
          result1 = result1 !== null ? result1 : "";
          if (result1 !== null) {
            result2 = parse_ws();
            if (result2 !== null) {
              result3 = [];
              pos2 = pos;
              pos3 = pos;
              result4 = parse_rule();
              if (result4 !== null) {
                result5 = parse_ret();
                if (result5 !== null) {
                  result4 = [result4, result5];
                } else {
                  result4 = null;
                  pos = pos3;
                }
              } else {
                result4 = null;
                pos = pos3;
              }
              if (result4 !== null) {
                result4 = (function(offset, rule) { return rule.join('') })(pos2, result4[0]);
              }
              if (result4 === null) {
                pos = pos2;
              }
              while (result4 !== null) {
                result3.push(result4);
                pos2 = pos;
                pos3 = pos;
                result4 = parse_rule();
                if (result4 !== null) {
                  result5 = parse_ret();
                  if (result5 !== null) {
                    result4 = [result4, result5];
                  } else {
                    result4 = null;
                    pos = pos3;
                  }
                } else {
                  result4 = null;
                  pos = pos3;
                }
                if (result4 !== null) {
                  result4 = (function(offset, rule) { return rule.join('') })(pos2, result4[0]);
                }
                if (result4 === null) {
                  pos = pos2;
                }
              }
              if (result3 !== null) {
                result4 = parse_ret();
                result4 = result4 !== null ? result4 : "";
                if (result4 !== null) {
                  result0 = [result0, result1, result2, result3, result4];
                } else {
                  result0 = null;
                  pos = pos1;
                }
              } else {
                result0 = null;
                pos = pos1;
              }
            } else {
              result0 = null;
              pos = pos1;
            }
          } else {
            result0 = null;
            pos = pos1;
          }
        } else {
          result0 = null;
          pos = pos1;
        }
        if (result0 !== null) {
          result0 = (function(offset, rules) { return rules })(pos0, result0[3]);
        }
        if (result0 === null) {
          pos = pos0;
        }
        return result0;
      }
      
      function parse_rule() {
        var result0, result1;
        
        result0 = [];
        result1 = parse_words();
        while (result1 !== null) {
          result0.push(result1);
          result1 = parse_words();
        }
        return result0;
      }
      
      function parse_words() {
        var result0, result1;
        var pos0;
        
        pos0 = pos;
        result1 = parse_word();
        if (result1 !== null) {
          result0 = [];
          while (result1 !== null) {
            result0.push(result1);
            result1 = parse_word();
          }
        } else {
          result0 = null;
        }
        if (result0 !== null) {
          result0 = (function(offset, words) { return words.join('') })(pos0, result0);
        }
        if (result0 === null) {
          pos = pos0;
        }
        return result0;
      }
      
      function parse_word() {
        var result0, result1;
        var pos0;
        
        pos0 = pos;
        if (/^[A-z0-9\xC6\xE1-\xEF\xC0-\xFF'"!?\-.,:;\u2010-\u2019\\{}+\/() ]/.test(input.charAt(pos))) {
          result1 = input.charAt(pos);
          pos++;
        } else {
          result1 = null;
          if (reportFailures === 0) {
            matchFailed("[A-z0-9\\xC6\\xE1-\\xEF\\xC0-\\xFF'\"!?\\-.,:;\\u2010-\\u2019\\\\{}+\\/() ]");
          }
        }
        if (result1 !== null) {
          result0 = [];
          while (result1 !== null) {
            result0.push(result1);
            if (/^[A-z0-9\xC6\xE1-\xEF\xC0-\xFF'"!?\-.,:;\u2010-\u2019\\{}+\/() ]/.test(input.charAt(pos))) {
              result1 = input.charAt(pos);
              pos++;
            } else {
              result1 = null;
              if (reportFailures === 0) {
                matchFailed("[A-z0-9\\xC6\\xE1-\\xEF\\xC0-\\xFF'\"!?\\-.,:;\\u2010-\\u2019\\\\{}+\\/() ]");
              }
            }
          }
        } else {
          result0 = null;
        }
        if (result0 !== null) {
          result0 = (function(offset, word) { return word.join('') })(pos0, result0);
        }
        if (result0 === null) {
          pos = pos0;
        }
        return result0;
      }
      
      function parse_leftparen() {
        var result0;
        
        if (input.charCodeAt(pos) === 40) {
          result0 = "(";
          pos++;
        } else {
          result0 = null;
          if (reportFailures === 0) {
            matchFailed("\"(\"");
          }
        }
        return result0;
      }
      
      function parse_rightparen() {
        var result0;
        
        if (input.charCodeAt(pos) === 41) {
          result0 = ")";
          pos++;
        } else {
          result0 = null;
          if (reportFailures === 0) {
            matchFailed("\")\"");
          }
        }
        return result0;
      }
      
      function parse_slash() {
        var result0;
        
        if (input.charCodeAt(pos) === 47) {
          result0 = "/";
          pos++;
        } else {
          result0 = null;
          if (reportFailures === 0) {
            matchFailed("\"/\"");
          }
        }
        return result0;
      }
      
      function parse_colon() {
        var result0;
        
        if (input.charCodeAt(pos) === 58) {
          result0 = ":";
          pos++;
        } else {
          result0 = null;
          if (reportFailures === 0) {
            matchFailed("\":\"");
          }
        }
        return result0;
      }
      
      function parse_dash() {
        var result0, result1;
        
        if (/^[\-\u2010-\u2015]/.test(input.charAt(pos))) {
          result1 = input.charAt(pos);
          pos++;
        } else {
          result1 = null;
          if (reportFailures === 0) {
            matchFailed("[\\-\\u2010-\\u2015]");
          }
        }
        if (result1 !== null) {
          result0 = [];
          while (result1 !== null) {
            result0.push(result1);
            if (/^[\-\u2010-\u2015]/.test(input.charAt(pos))) {
              result1 = input.charAt(pos);
              pos++;
            } else {
              result1 = null;
              if (reportFailures === 0) {
                matchFailed("[\\-\\u2010-\\u2015]");
              }
            }
          }
        } else {
          result0 = null;
        }
        return result0;
      }
      
      function parse_number() {
        var result0, result1, result2, result3;
        var pos0, pos1;
        
        if (input.charCodeAt(pos) === 48) {
          result0 = "0";
          pos++;
        } else {
          result0 = null;
          if (reportFailures === 0) {
            matchFailed("\"0\"");
          }
        }
        if (result0 === null) {
          if (input.charCodeAt(pos) === 42) {
            result0 = "*";
            pos++;
          } else {
            result0 = null;
            if (reportFailures === 0) {
              matchFailed("\"*\"");
            }
          }
          if (result0 === null) {
            pos0 = pos;
            if (/^[1-9]/.test(input.charAt(pos))) {
              result1 = input.charAt(pos);
              pos++;
            } else {
              result1 = null;
              if (reportFailures === 0) {
                matchFailed("[1-9]");
              }
            }
            if (result1 !== null) {
              result0 = [];
              while (result1 !== null) {
                result0.push(result1);
                if (/^[1-9]/.test(input.charAt(pos))) {
                  result1 = input.charAt(pos);
                  pos++;
                } else {
                  result1 = null;
                  if (reportFailures === 0) {
                    matchFailed("[1-9]");
                  }
                }
              }
            } else {
              result0 = null;
            }
            if (result0 !== null) {
              result1 = [];
              if (/^[0-9]/.test(input.charAt(pos))) {
                result2 = input.charAt(pos);
                pos++;
              } else {
                result2 = null;
                if (reportFailures === 0) {
                  matchFailed("[0-9]");
                }
              }
              while (result2 !== null) {
                result1.push(result2);
                if (/^[0-9]/.test(input.charAt(pos))) {
                  result2 = input.charAt(pos);
                  pos++;
                } else {
                  result2 = null;
                  if (reportFailures === 0) {
                    matchFailed("[0-9]");
                  }
                }
              }
              if (result1 !== null) {
                if (/^[+\-]/.test(input.charAt(pos))) {
                  result2 = input.charAt(pos);
                  pos++;
                } else {
                  result2 = null;
                  if (reportFailures === 0) {
                    matchFailed("[+\\-]");
                  }
                }
                if (result2 !== null) {
                  if (input.charCodeAt(pos) === 42) {
                    result3 = "*";
                    pos++;
                  } else {
                    result3 = null;
                    if (reportFailures === 0) {
                      matchFailed("\"*\"");
                    }
                  }
                  if (result3 !== null) {
                    result0 = [result0, result1, result2, result3];
                  } else {
                    result0 = null;
                    pos = pos0;
                  }
                } else {
                  result0 = null;
                  pos = pos0;
                }
              } else {
                result0 = null;
                pos = pos0;
              }
            } else {
              result0 = null;
              pos = pos0;
            }
            if (result0 === null) {
              pos0 = pos;
              pos1 = pos;
              if (/^[\-]/.test(input.charAt(pos))) {
                result0 = input.charAt(pos);
                pos++;
              } else {
                result0 = null;
                if (reportFailures === 0) {
                  matchFailed("[\\-]");
                }
              }
              result0 = result0 !== null ? result0 : "";
              if (result0 !== null) {
                if (/^[1-9]/.test(input.charAt(pos))) {
                  result2 = input.charAt(pos);
                  pos++;
                } else {
                  result2 = null;
                  if (reportFailures === 0) {
                    matchFailed("[1-9]");
                  }
                }
                if (result2 !== null) {
                  result1 = [];
                  while (result2 !== null) {
                    result1.push(result2);
                    if (/^[1-9]/.test(input.charAt(pos))) {
                      result2 = input.charAt(pos);
                      pos++;
                    } else {
                      result2 = null;
                      if (reportFailures === 0) {
                        matchFailed("[1-9]");
                      }
                    }
                  }
                } else {
                  result1 = null;
                }
                if (result1 !== null) {
                  result2 = [];
                  if (/^[0-9]/.test(input.charAt(pos))) {
                    result3 = input.charAt(pos);
                    pos++;
                  } else {
                    result3 = null;
                    if (reportFailures === 0) {
                      matchFailed("[0-9]");
                    }
                  }
                  while (result3 !== null) {
                    result2.push(result3);
                    if (/^[0-9]/.test(input.charAt(pos))) {
                      result3 = input.charAt(pos);
                      pos++;
                    } else {
                      result3 = null;
                      if (reportFailures === 0) {
                        matchFailed("[0-9]");
                      }
                    }
                  }
                  if (result2 !== null) {
                    result0 = [result0, result1, result2];
                  } else {
                    result0 = null;
                    pos = pos1;
                  }
                } else {
                  result0 = null;
                  pos = pos1;
                }
              } else {
                result0 = null;
                pos = pos1;
              }
              if (result0 !== null) {
                result0 = (function(offset, minus, number) { return minus + parseInt(number.join(''), 10); })(pos0, result0[0], result0[1]);
              }
              if (result0 === null) {
                pos = pos0;
              }
            }
          }
        }
        return result0;
      }
      
      function parse_ws() {
        var result0, result1;
        
        if (/^[,\t ]/.test(input.charAt(pos))) {
          result1 = input.charAt(pos);
          pos++;
        } else {
          result1 = null;
          if (reportFailures === 0) {
            matchFailed("[,\\t ]");
          }
        }
        if (result1 !== null) {
          result0 = [];
          while (result1 !== null) {
            result0.push(result1);
            if (/^[,\t ]/.test(input.charAt(pos))) {
              result1 = input.charAt(pos);
              pos++;
            } else {
              result1 = null;
              if (reportFailures === 0) {
                matchFailed("[,\\t ]");
              }
            }
          }
        } else {
          result0 = null;
        }
        return result0;
      }
      
      function parse_space() {
        var result0, result1;
        
        if (/^[ ]/.test(input.charAt(pos))) {
          result1 = input.charAt(pos);
          pos++;
        } else {
          result1 = null;
          if (reportFailures === 0) {
            matchFailed("[ ]");
          }
        }
        if (result1 !== null) {
          result0 = [];
          while (result1 !== null) {
            result0.push(result1);
            if (/^[ ]/.test(input.charAt(pos))) {
              result1 = input.charAt(pos);
              pos++;
            } else {
              result1 = null;
              if (reportFailures === 0) {
                matchFailed("[ ]");
              }
            }
          }
        } else {
          result0 = null;
        }
        return result0;
      }
      
      function parse_ret() {
        var result0, result1;
        
        if (/^[\n\r]/.test(input.charAt(pos))) {
          result1 = input.charAt(pos);
          pos++;
        } else {
          result1 = null;
          if (reportFailures === 0) {
            matchFailed("[\\n\\r]");
          }
        }
        if (result1 !== null) {
          result0 = [];
          while (result1 !== null) {
            result0.push(result1);
            if (/^[\n\r]/.test(input.charAt(pos))) {
              result1 = input.charAt(pos);
              pos++;
            } else {
              result1 = null;
              if (reportFailures === 0) {
                matchFailed("[\\n\\r]");
              }
            }
          }
        } else {
          result0 = null;
        }
        return result0;
      }
      
      
      function cleanupExpected(expected) {
        expected.sort();
        
        var lastExpected = null;
        var cleanExpected = [];
        for (var i = 0; i < expected.length; i++) {
          if (expected[i] !== lastExpected) {
            cleanExpected.push(expected[i]);
            lastExpected = expected[i];
          }
        }
        return cleanExpected;
      }
      
      function computeErrorPosition() {
        /*
         * The first idea was to use |String.split| to break the input up to the
         * error position along newlines and derive the line and column from
         * there. However IE's |split| implementation is so broken that it was
         * enough to prevent it.
         */
        
        var line = 1;
        var column = 1;
        var seenCR = false;
        
        for (var i = 0; i < Math.max(pos, rightmostFailuresPos); i++) {
          var ch = input.charAt(i);
          if (ch === "\n") {
            if (!seenCR) { line++; }
            column = 1;
            seenCR = false;
          } else if (ch === "\r" || ch === "\u2028" || ch === "\u2029") {
            line++;
            column = 1;
            seenCR = true;
          } else {
            column++;
            seenCR = false;
          }
        }
        
        return { line: line, column: column };
      }
      
      
      var result = parseFunctions[startRule]();
      
      /*
       * The parser is now in one of the following three states:
       *
       * 1. The parser successfully parsed the whole input.
       *
       *    - |result !== null|
       *    - |pos === input.length|
       *    - |rightmostFailuresExpected| may or may not contain something
       *
       * 2. The parser successfully parsed only a part of the input.
       *
       *    - |result !== null|
       *    - |pos < input.length|
       *    - |rightmostFailuresExpected| may or may not contain something
       *
       * 3. The parser did not successfully parse any part of the input.
       *
       *   - |result === null|
       *   - |pos === 0|
       *   - |rightmostFailuresExpected| contains at least one failure
       *
       * All code following this comment (including called functions) must
       * handle these states.
       */
      if (result === null || pos !== input.length) {
        var offset = Math.max(pos, rightmostFailuresPos);
        var found = offset < input.length ? input.charAt(offset) : null;
        var errorPosition = computeErrorPosition();
        
        throw new this.SyntaxError(
          cleanupExpected(rightmostFailuresExpected),
          found,
          offset,
          errorPosition.line,
          errorPosition.column
        );
      }
      
      return result;
    },
    
    /* Returns the parser source code. */
    toSource: function() { return this._source; }
  };
  
  /* Thrown when a parser encounters a syntax error. */
  
  result.SyntaxError = function(expected, found, offset, line, column) {
    function buildMessage(expected, found) {
      var expectedHumanized, foundHumanized;
      
      switch (expected.length) {
        case 0:
          expectedHumanized = "end of input";
          break;
        case 1:
          expectedHumanized = expected[0];
          break;
        default:
          expectedHumanized = expected.slice(0, expected.length - 1).join(", ")
            + " or "
            + expected[expected.length - 1];
      }
      
      foundHumanized = found ? quote(found) : "end of input";
      
      return "Expected " + expectedHumanized + " but " + foundHumanized + " found.";
    }
    
    this.name = "SyntaxError";
    this.expected = expected;
    this.found = found;
    this.message = buildMessage(expected, found);
    this.offset = offset;
    this.line = line;
    this.column = column;
  };
  
  result.SyntaxError.prototype = Error.prototype;
  
  return result;
})();
